/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:	Dean Johnson (dtj@sgi.com)
 *
 * Purpose:	A simple self-destructive splash screen panel.
 *
 */

#include "AboutPanel.h"
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "hoover.h"
#include "hoover_splash.xpm"

AboutPanel::AboutPanel(char *fullpath) 
{
	GtkWidget *p = NULL;
	GdkColor white = { 0, 0xffff, 0xffff, 0xffff };
	GtkStyle *style;
	GdkPixmap *pix = NULL;
	GdkBitmap *mask = NULL;

	gdk_colormap_alloc_color(gdk_colormap_get_system(),&white,FALSE,TRUE);
	style = gtk_style_copy(gtk_widget_get_default_style());
	style->bg[GTK_STATE_NORMAL] = white;

//	setSelfDestruct();

	theBaseWidget = gtk_table_new(2 /*rows*/,1 /*cols*/,FALSE);
	show();

	theContentWidget = gtk_fixed_new();
	gtk_widget_set_style(theContentWidget,style);
	gtk_container_add(GTK_CONTAINER(theBaseWidget), theContentWidget);

	if (fullpath != NULL) {
		struct stat s;
		int res = stat(fullpath, &s);
		if (!res && S_ISREG(s.st_mode)) {
			// it exists!

			GdkColormap *colormap = gtk_widget_get_colormap(theContentWidget);
			pix = gdk_pixmap_colormap_create_from_xpm(NULL,colormap,
						&mask,NULL,fullpath);
			if (pix != NULL) {
				p = gtk_pixmap_new(pix,mask);
				gdk_pixmap_unref(pix);
				gdk_bitmap_unref(mask);
			}
		}
	}
	if (p == NULL) {
		p = getIcon(hoover_splash_xpm);
	}
	gtk_fixed_put(GTK_FIXED(theContentWidget), p, 40, 0);
        gtk_widget_show_all(theContentWidget);
}

