#include <stdio.h>
#include <assert.h>
#include <gtk/gtk.h>
#include "hoover.h"
#include "node.h"
#include "input_queue.h"
#include "question.xpm"
#include "dialogs.h"
#include "libgessie/Utils.h"

class AdminDialogSquirrel;

static void handleAddrRules(InputEntry *entry,gpointer data);
static void handleModuleRules(InputEntry *entry,gpointer data);
static void doAdminAction(gpointer data,GtkWidget *w);
static void doRuleAddDialog(AdminDialogSquirrel *ads,gboolean isAddress);

#define	ENTRY_FUNC(a) void a(InputEntry *entry,gpointer data)

#define BUTTON_PAD	1

#define MK_BUTTON(par,nm,str,cb,n)					\
	nm = gtk_button_new_with_label(str);				\
	gtk_container_set_border_width (GTK_CONTAINER (nm), 2);		\
	gtk_box_pack_start(GTK_BOX(par),nm,FALSE,FALSE,BUTTON_PAD);	\
	gtk_widget_show(nm);						\
	gtk_widget_set_sensitive(nm,FALSE);				\
	gtk_signal_connect_object(GTK_OBJECT(nm), "clicked",		\
				  GTK_SIGNAL_FUNC(cb),(GtkObject *) n)

#define	ADD_TO_TABLE(a)		\
	gtk_table_attach(GTK_TABLE(table),a,0,1,vPos,vPos+1,	\
			 (enum GtkAttachOptions)(GTK_FILL | GTK_EXPAND), \
			 (enum GtkAttachOptions)(GTK_FILL | GTK_EXPAND),0,0)

#define	ADD_TO_TABLE2(a,b)	\
	gtk_table_attach(GTK_TABLE(table),a,b,b+1,vPos,vPos+1,	\
			 (enum GtkAttachOptions)(GTK_FILL | GTK_EXPAND), \
			 (enum GtkAttachOptions)(GTK_FILL | GTK_EXPAND),0,0)

#define	ADD_TO_TABLE3(a,b,c)	\
	gtk_table_attach(GTK_TABLE(table),a,b,c,vPos,vPos+1,	\
			 (enum GtkAttachOptions)(GTK_FILL | GTK_EXPAND), \
			 (enum GtkAttachOptions)(GTK_FILL | GTK_EXPAND),0,0)

#define	SET_DATA	gtk_object_set_data
#define	GET_DATA(a,b)	gtk_object_get_data(GTK_OBJECT(a),b)

class AddressRule {
public:
	AddressRule(char *p,char *a,char *s) {
		policy = strdup(p);
		address = strdup(a);
		sigbits = strdup(s);
	}
	char	*policy;
	char	*address;
	char	*sigbits;
};

class AdminDialogSquirrel {
public:
	AdminDialogSquirrel() {
		theSelectedAdmin = NULL;
		theSelectedRule  = NULL;
	}
	ConnectInfo	*connection;
	gboolean	being_destroyed;

	GtkWidget	*dialog;	// outside dialog

	GtkWidget	*admins;	// admin list
	GtkWidget	*add_admin;	// add an admin entry
	GtkWidget	*del_admin;	// delete an admin entry

	GtkWidget	*tree;		// rules tree
	GtkWidget	*add_mod_rule;
	GtkWidget	*add_addr_rule;
	GtkWidget	*del_rule;

	GtkWidget	*theSelectedAdmin;
	GtkWidget	*theSelectedRule;
	void	clearRulesList() {
		if (being_destroyed) return;
		gtk_tree_clear_items((GtkTree *) tree,0,-1);
	}
	void	clearAdminList() {
		if (being_destroyed) return;
		gtk_list_clear_items((GtkList *) admins,0,-1);
	}
};

class	AdminInfo {
public:
	AdminInfo(char *adm,GtkWidget *t,AdminDialogSquirrel *sq) {
		admin = strdup(adm);
		tree = t;
		ads = sq;
	}
	char	*admin;
	GtkWidget *tree;
	AdminDialogSquirrel *ads;
};

#define	MODULE_RULE_SELECTED	1
#define	ADDRESS_RULE_SELECTED	2
#define	NO_ADMIN_SELECTED	3
#define	YES_ADMIN_SELECTED	4
#define	RULE_UNSELECTED		5
#define	ADMINS_LISTED		6

static void
setSensitivity(AdminDialogSquirrel *ads,int signal)
{
	if (ads->being_destroyed) return;
	switch (signal) {
	case MODULE_RULE_SELECTED:
		gtk_widget_set_sensitive(ads->del_rule,		TRUE);
		break;
	case ADDRESS_RULE_SELECTED:
		gtk_widget_set_sensitive(ads->del_rule,		TRUE);
		break;
	case RULE_UNSELECTED:
		gtk_widget_set_sensitive(ads->del_rule,		FALSE);
		break;
	case NO_ADMIN_SELECTED:
		gtk_widget_set_sensitive(ads->add_addr_rule,	FALSE);
		gtk_widget_set_sensitive(ads->add_mod_rule,	FALSE);
		gtk_widget_set_sensitive(ads->del_rule,		FALSE);
		gtk_widget_set_sensitive(ads->del_admin,	FALSE);
		break;
	case YES_ADMIN_SELECTED:
		gtk_widget_set_sensitive(ads->add_addr_rule,	TRUE);
		gtk_widget_set_sensitive(ads->add_mod_rule,	TRUE);
		gtk_widget_set_sensitive(ads->del_admin,	TRUE);
		break;
	case ADMINS_LISTED:
		gtk_widget_set_sensitive(ads->add_admin,	TRUE);
		break;
	}
}

static void
ruleTreeAction(GtkWidget *w,gpointer data)
{
	AdminInfo *info = (AdminInfo *) GET_DATA(w,"admin");
	int	flag = (int) data;

	setSensitivity(info->ads,flag);
	if (flag == MODULE_RULE_SELECTED || flag == ADDRESS_RULE_SELECTED) {
		info->ads->theSelectedRule = w;
	}
	else	{
		info->ads->theSelectedRule = NULL;
	}
}

static
ENTRY_FUNC(handleModuleRules)
{
	AdminInfo *info = (AdminInfo *) data;
	GtkWidget *item;
	char	*cp1;
	char	*cp2;
	char	*cp3;
	char	buf[1024];

	for (InputEntry *tmp = entry; tmp; tmp = tmp->next) {
		(void) strtok(tmp->string,":");		// skip junk
		cp1 = strtok(NULL,":");			// the module
		cp2 = strtok(NULL,":");			// the policy
		cp3 = strtok(NULL,":");			// the glob

		sprintf(buf,"%s:%s (%s)",cp1,cp3,cp2);
		item = gtk_tree_item_new_with_label(buf);
		gtk_tree_append(GTK_TREE(info->tree),item);
		gtk_widget_show(item);

		SET_DATA(GTK_OBJECT(item),"policy",
				    (gpointer) strdup(cp2));
		SET_DATA(GTK_OBJECT(item),"module",
				    (gpointer) strdup(cp1));
		SET_DATA(GTK_OBJECT(item),"glob",  
				    (gpointer) strdup(cp3));
		SET_DATA(GTK_OBJECT(item),"admin", (gpointer) info);
		gtk_signal_connect(GTK_OBJECT(item), "select", 
				   GTK_SIGNAL_FUNC(ruleTreeAction), 
				   (gpointer) MODULE_RULE_SELECTED);
		gtk_signal_connect(GTK_OBJECT(item), "deselect", 
				   GTK_SIGNAL_FUNC(ruleTreeAction), 
				   (gpointer) RULE_UNSELECTED);
	}
}

static
ENTRY_FUNC(handleAddrRules)
{
	AdminInfo *info = (AdminInfo *) data;
	GtkWidget *item;
	char	buf[1024];
	char	*cp1;
	char	*cp2;
	char	*cp3;

	for (InputEntry *tmp = entry; tmp; tmp = tmp->next) {
                (void) strtok(tmp->string,":");		// skip junk
                cp1 = strtok(NULL,":");			// the policy
                cp2 = strtok(NULL,":");			// the addr
                cp3 = strtok(NULL,":");			// the sig bits

		sprintf(buf,"%s:%s (%s)",cp2,cp3,cp1);
		item = gtk_tree_item_new_with_label(buf);
		SET_DATA(GTK_OBJECT(item),"policy", 
				    (gpointer) strdup(cp1));
		SET_DATA(GTK_OBJECT(item),"addr",   
				    (gpointer) strdup(cp2));
		SET_DATA(GTK_OBJECT(item),"sigbits",
				    (gpointer) strdup(cp3));
		SET_DATA(GTK_OBJECT(item),"admin",  (gpointer) info);
		gtk_tree_append(GTK_TREE(info->tree),item);
		gtk_widget_show(item);
		gtk_signal_connect(GTK_OBJECT(item), "select", 
				   GTK_SIGNAL_FUNC(ruleTreeAction), 
				   (gpointer) ADDRESS_RULE_SELECTED);
		gtk_signal_connect(GTK_OBJECT(item), "deselect", 
				   GTK_SIGNAL_FUNC(ruleTreeAction),
				   (gpointer) RULE_UNSELECTED);
	}
}

static void
fillRuleList(AdminDialogSquirrel *ads,char *admin)
{
	GtkWidget *item;
	GtkWidget *subtree;
	char	buf[1024];
	
	gtk_tree_clear_items((GtkTree *) ads->tree,0,-1);

	/*--------------------------------------*
	 *		MODULE RULES		*
	 *--------------------------------------*/

	item = gtk_tree_item_new_with_label("Module Rules");
	NONSELECT(item);
	gtk_tree_append(GTK_TREE(ads->tree),item);
	gtk_widget_show(item);

	subtree = gtk_tree_new();
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(item),subtree);
	sprintf(buf,"NEXXUS:ADMIN_LIST_MOD_ACL_RULES:%s:",admin);
	ads->connection->sendMsg(buf,TRUE,handleModuleRules,
		     (gpointer) new AdminInfo(admin,subtree,ads));
	gtk_tree_item_expand((GtkTreeItem *) item);

	/*--------------------------------------*
	 *		 ADDR RULES		*
	 *--------------------------------------*/

	item = gtk_tree_item_new_with_label("Address Rules");
	NONSELECT(item);
	gtk_tree_append(GTK_TREE(ads->tree),item);
	gtk_widget_show(item);

	subtree = gtk_tree_new();
	gtk_tree_item_set_subtree(GTK_TREE_ITEM(item),subtree);
	sprintf(buf,"NEXXUS:ADMIN_LIST_ADDR_ACL_RULES:%s:",admin);
	ads->connection->sendMsg(buf,TRUE,handleAddrRules,
		     (gpointer) new AdminInfo(admin,subtree,ads));
	gtk_tree_item_expand((GtkTreeItem *) item);

}

static void
gotoAdmin(gpointer data,GtkWidget *w)
{
	AdminDialogSquirrel *ads = (AdminDialogSquirrel *) data;
	char *admin = (char *) GET_DATA(w,"admin");
	fillRuleList(ads,admin);
	setSensitivity(ads,YES_ADMIN_SELECTED);
	ads->theSelectedAdmin = w;
}

static void
unGotoAdmin(gpointer data,GtkWidget * /*w*/)
{
	AdminDialogSquirrel *ads = (AdminDialogSquirrel *) data;

	ads->clearRulesList();
	setSensitivity(ads,NO_ADMIN_SELECTED);
	ads->theSelectedAdmin = NULL;
}

static
ENTRY_FUNC(handleAdminList)
{
	AdminDialogSquirrel	*ads = (AdminDialogSquirrel *) data;
	GtkWidget *item;
	char	*admin;
	GList	*admins = NULL;
	char	*block;

	for (InputEntry *tmp = entry; tmp; tmp = tmp->next) {
                (void) strtok(tmp->string,":");		// skip ADMIN
                admin = strtok(NULL,":");		// the state
		block = strdup(admin);
		item = gtk_list_item_new_with_label(strdup(admin));
		gtk_widget_show(item);
		SET_DATA(GTK_OBJECT(item),"admin",(void *) block);
		admins = g_list_append(admins,(gpointer) item);
	}
	gtk_list_append_items((GtkList *) ads->admins,admins);
	setSensitivity(ads,ADMINS_LISTED);
}

static void
setupTree(AdminDialogSquirrel *ads,gboolean clear_first) 
{
	if (clear_first) {
		ads->clearRulesList();
		ads->clearAdminList();
	}
	ads->connection->sendMsg("NEXXUS:ADMIN_LIST:",TRUE,handleAdminList,(gpointer) ads);
}

static
ENTRY_FUNC(refreshEverything)
{
	AdminDialogSquirrel *ads = (AdminDialogSquirrel *) data;

	setupTree(ads,TRUE);
}

static void
junkAction(GtkWidget *,gpointer)
{
	// do nothing
}

static void
addAdminName(char *name,char *pw,char *pw2,gpointer clientData,
	     gpointer /*callData*/)
{
	AdminDialogSquirrel *ads = (AdminDialogSquirrel *) clientData;
	char	buf[1024];

	if (strcmp(pw,pw2)) {	// given passwords don't match
		binary_dialog("Passwords Don't Match!   \nTry Again?",
			      GTK_SIGNAL_FUNC(doAdminAction),
			      GTK_SIGNAL_FUNC(junkAction),clientData);
		return;
	}
	//	send the name
	sprintf(buf,"NEXXUS:ADMIN_ADD:%s:%s:default:",name,pw);
	ads->connection->sendMsg(buf,TRUE,refreshEverything,(gpointer) ads);
}

static void
doAdminAction(gpointer data,GtkWidget *w)
{
	AdminDialogSquirrel *ads = (AdminDialogSquirrel *) data;

	if (w == ads->del_admin) {
		char	buf[1024];
		if (ads->theSelectedAdmin != NULL) {
			//	remove item selected in ads->admins
			char *block = (char *) GET_DATA(ads->theSelectedAdmin,
							"admin");
			sprintf(buf,"NEXXUS:ADMIN_DEL:%s:",block);
			ads->connection->sendMsg(buf,TRUE,refreshEverything,
						 (gpointer) ads);
		}
	}
	else	{
		/*
		 *	Default to adding an admin
		 */
		get_passwordentry_dialog("Add Admin to List",addAdminName,
					 NULL,"Password again",NULL,
				     	 (gpointer) ads);
	}
}

static void
nukeTheRule(AdminDialogSquirrel *ads)
{
	AdminInfo *info;
	GtkWidget *item = ads->theSelectedRule;
	char	*policy;
	char	buf[1024];

	if (item == NULL) {
		//	should do error message
		return;
	}
	info   = (AdminInfo *) GET_DATA(item,"admin");
	policy = (char *)      GET_DATA(item,"policy");

	char *module = (char *) GET_DATA(item,"module");
	if (module == NULL) {	// its an address acl
		// :ADMIN_DEL_ADDR_ACL_RULE:<ADMIN>:<POLICY>:<ADDR>:<SIGBITS>
		char *addr    = (char *) GET_DATA(item,"addr");
		char *sigbits = (char *) GET_DATA(item,"sigbits");
		sprintf(buf,"NEXXUS:ADMIN_DEL_ADDR_ACL_RULE:%s:%s:%s:%s:",
			info->admin,policy,addr,sigbits);
	}
	else	{
		// :ADMIN_DEL_MOD_ACL_RULE:<ADMIN>:<POLICY>:<MODULE>:<GLOB>
		char *glob = (char *) GET_DATA(item,"glob");
		sprintf(buf,"NEXXUS:ADMIN_DEL_MOD_ACL_RULE:%s:%s:%s:%s:",
			info->admin,policy,module,glob);
	}
	ads->connection->sendMsg(buf,TRUE,refreshEverything,(gpointer) ads);
}

static void
doRuleAction(gpointer data,GtkWidget *w)
{
	AdminDialogSquirrel *ads = (AdminDialogSquirrel *) data;

	if (w == ads->del_rule) {
		nukeTheRule(ads);
	}
	else if (w == ads->add_mod_rule) {
		doRuleAddDialog(ads,FALSE);
	}
	else if (w == ads->add_addr_rule) {
		doRuleAddDialog(ads,TRUE);
	}
}

static void
adminDone(gpointer data,GtkWidget *)
{
	AdminDialogSquirrel *tmp = (AdminDialogSquirrel *) data;
	GtkWidget *dialog = tmp->dialog;

	tmp->being_destroyed = TRUE;
	gtk_widget_destroy(dialog);
	free(tmp);
}

void
doAdminDialog(gpointer callback_data,GtkWidget *)
{
	int	vPos = 0;
	AdminDialogSquirrel	*tmp;
	ConnectInfo *con = (ConnectInfo *) callback_data;
	char	buf[1024];
	GtkWidget *sep;

	/* Create the widgets */

	assert(con != NULL);

	/*
	 *	Stash away some stuff for later
	 */
	tmp = new AdminDialogSquirrel();
	tmp->connection	= con;
	tmp->being_destroyed = FALSE;

	GtkWidget *dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	tmp->dialog = dialog;

	/*------------------------------------------------------*
	 *			Top portion			*
	 *------------------------------------------------------*/

        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	GtkWidget *icon = getIcon((gchar **) question_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	sprintf(buf,"Cluster '%s'\nAuthority Management",con->hostname);
	GtkWidget *label = gtk_label_new(buf);
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*------------------------------------------------------*
	 *			Data portion			*
	 *------------------------------------------------------*/

        GtkWidget *table = gtk_table_new(7 /*rows*/,2 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) table,5);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	sep = gtk_hseparator_new();
	gtk_widget_show(sep);
	ADD_TO_TABLE(sep);
	vPos++;

	label = gtk_label_new("Admin List");
	gtk_widget_show(label);
	ADD_TO_TABLE(label);
	vPos++;

	/*------------------------------------------------------*
	 *			scrolled list widget		*
	 *------------------------------------------------------*/

	GtkWidget *scrollw1 = gtk_scrolled_window_new((GtkAdjustment *) NULL,
						      (GtkAdjustment *) NULL);
	gtk_widget_set_usize(scrollw1,275,125);
	gtk_widget_show(scrollw1);
	assert(scrollw1 != NULL);

	tmp->admins = gtk_list_new();
	gtk_widget_set_usize(tmp->admins,250,75);
	gtk_widget_show(tmp->admins);
	gtk_signal_connect_object(GTK_OBJECT(tmp->admins),"select_child",
				  GTK_SIGNAL_FUNC(gotoAdmin),
				  (GtkObject *) tmp);
	gtk_signal_connect_object(GTK_OBJECT(tmp->admins),"unselect_child",
				  GTK_SIGNAL_FUNC(unGotoAdmin),
				  (GtkObject *) tmp);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw1),
					      tmp->admins);
	ADD_TO_TABLE(scrollw1);
	vPos++;

	/*------------------------------------------------------*
	 *		    Button portion, part 1		*
	 *------------------------------------------------------*/

        GtkWidget *hbox1 = gtk_hbox_new(FALSE /*TRUE*/, 3);
        gtk_container_set_border_width (GTK_CONTAINER (hbox1), 0);
        gtk_widget_show (hbox1);
        gtk_table_attach(GTK_TABLE(table),hbox1,0,2,vPos,vPos+1,
                         (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
                         GTK_SHRINK, 0,0);

	MK_BUTTON(hbox1,tmp->add_admin,"Add Admin",   doAdminAction,tmp);
	MK_BUTTON(hbox1,tmp->del_admin,"Delete Admin",doAdminAction,tmp);
	vPos++;

	/*
	 *-----------------------------------------------------
	 */

	sep = gtk_hseparator_new();
	gtk_widget_show(sep);
	ADD_TO_TABLE(sep);
	vPos++;

	label = gtk_label_new("Rules List");
	gtk_widget_show(label);
	ADD_TO_TABLE(label);
	vPos++;

	/*
	 *	scroll tree widget
	 */

	GtkWidget *scrollw2 = gtk_scrolled_window_new((GtkAdjustment *) NULL,
						      (GtkAdjustment *) NULL);
	gtk_widget_set_usize(scrollw2,275,125);

	gtk_widget_show(scrollw2);
	assert(scrollw2 != NULL);

	tmp->tree = (GtkWidget *) gtk_tree_new();
	gtk_tree_set_selection_mode(GTK_TREE(tmp->tree),GTK_SELECTION_SINGLE);
	gtk_widget_show(tmp->tree);

	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw2),
					      tmp->tree);
	ADD_TO_TABLE(scrollw2);
	vPos++;

	/*------------------------------------------------------*
	 *		    Button portion, part 2		*
	 *------------------------------------------------------*/

        GtkWidget *hbox2 = gtk_hbox_new(FALSE /*TRUE*/, 3);
        gtk_container_set_border_width (GTK_CONTAINER (hbox2), 0);
        gtk_widget_show (hbox2);
        gtk_table_attach(GTK_TABLE(table),hbox2,0,2,vPos,vPos+1,
                         (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
                         GTK_SHRINK, 0,0);

	MK_BUTTON(hbox2,tmp->add_addr_rule,"Add Address Rule",doRuleAction,tmp);
	MK_BUTTON(hbox2,tmp->add_mod_rule ,"Add Module Rule", doRuleAction,tmp);
	MK_BUTTON(hbox2,tmp->del_rule,     "Delete",	      doRuleAction,tmp);

	/*------------------------------------------------------*
	 *		    Button portion, part 2		*
	 *------------------------------------------------------*/

	GtkWidget *done_button = gtk_button_new_with_label("Done");
        gtk_signal_connect_object(GTK_OBJECT (done_button), "clicked",
                                  GTK_SIGNAL_FUNC (adminDone),
                                  (GtkObject *) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  done_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all(dialog);

	setupTree(tmp,FALSE);
}

/*----------------------------------------------------------------------*
 *				ADDING DIALOGS				*
 *----------------------------------------------------------------------*/

typedef struct {
	AdminDialogSquirrel *ads;
	GtkWidget *dialog;
	GtkWidget *field1;
	GtkWidget *field2;
	gboolean  allow;
	gboolean  addr_rule;	// is this data for an addr rule?
} RuleAddSquirrel;

static void
setPolicyAllow(gpointer data,GtkWidget *)
{
	RuleAddSquirrel *tmp = (RuleAddSquirrel *) data;
	tmp->allow = TRUE;
}

static void
setPolicyDeny(gpointer data,GtkWidget *)
{
	RuleAddSquirrel *tmp = (RuleAddSquirrel *) data;
	tmp->allow = FALSE;
}

static void
gottaRuleCancel(GtkWidget *,gpointer data)
{
	RuleAddSquirrel *tmp = (RuleAddSquirrel *) data;

	GtkWidget *dialog = tmp->dialog;
	free(tmp);
	gtk_widget_destroy(dialog);
}

static void
gottaRuleApply(gpointer data, GtkWidget *)
{
	char	buf[1024];
	RuleAddSquirrel *tmp = (RuleAddSquirrel *) data;

	assert(tmp->ads != NULL);
	assert(tmp->ads->theSelectedAdmin != NULL);
	char *block = (char *) GET_DATA(tmp->ads->theSelectedAdmin,"admin");

	char *f1 = strdup((tmp->field1 != NULL) 
			? gtk_entry_get_text((GtkEntry *) tmp->field1) : "");
	char *f2 = strdup((tmp->field2 != NULL) 
			? gtk_entry_get_text((GtkEntry *) tmp->field2) : "");

	if (tmp->addr_rule) {		// it's an address rule!
		// :ADMIN_ADD_ADDR_ACL_RULE:<ADMIN>:<POLICY>:<ADDR>:<SIGBITS>
		sprintf(buf,"NEXXUS:ADMIN_ADD_ADDR_ACL_RULE:%s:%s:%s:%s:",
			block,(tmp->allow) ? "ALLOW":"DENY",f1,f2);
	}
	else	{			// it's a module rule!
		// :ADMIN_ADD_MOD_ACL_RULE:<ADMIN>:<POLICY>:<MODULE>:<GLOB>
		sprintf(buf,"NEXXUS:ADMIN_ADD_MOD_ACL_RULE:%s:%s:%s:%s:",
			block,(tmp->allow) ? "ALLOW":"DENY",f1,f2);
	}
	tmp->ads->connection->sendMsg(buf,TRUE,refreshEverything,
				      (gpointer) tmp->ads);
	gtk_widget_destroy(tmp->dialog);
	free(tmp);		// clean up
}

/*----------------------------------------------------------------------*
 *				ADDR DIALOG				*
 *----------------------------------------------------------------------*/

static void
doRuleAddDialog(AdminDialogSquirrel *ads,gboolean isAddress)
{
	int	vPos = 0;
	GtkWidget	*label;

	/* Create the widgets */

	GtkWidget *dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	/*------------------------------------------------------*
	 *			Top portion			*
	 *------------------------------------------------------*/

        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	GtkWidget *icon = getIcon((gchar **) question_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	label = gtk_label_new(isAddress ? "Add Address ACL Rule" 
					: "Add Module ACL Rule");
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*------------------------------------------------------*
	 *			Data portion			*
	 *------------------------------------------------------*/

	RuleAddSquirrel *tmp = (RuleAddSquirrel *) malloc(sizeof(RuleAddSquirrel));
	tmp->addr_rule = isAddress;
	tmp->ads = ads;
	tmp->dialog = dialog;
	
        GtkWidget *table = gtk_table_new(2 /*rows*/,4 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) table,5);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);

	label = gtk_label_new(isAddress ? "Address" : "Module");
	ADD_TO_TABLE2(label,0);

	tmp->field1 = gtk_entry_new_with_max_length(16);
	gtk_widget_set_usize(tmp->field1,100,-1);
	ADD_TO_TABLE2(tmp->field1,1);

	if (isAddress) {
		gtk_entry_set_text(GTK_ENTRY(tmp->field1), "192.168.0.0");

		ADD_TO_TABLE2(gtk_label_new(" / "),2);

		tmp->field2 = gtk_entry_new_with_max_length(2);
		gtk_entry_set_text(GTK_ENTRY(tmp->field2), "24");
		gtk_widget_set_usize(tmp->field2,35,-1);
		ADD_TO_TABLE2(tmp->field2,3);
	}
	else	{
		vPos++;

		ADD_TO_TABLE2(gtk_label_new("Command Pattern"),0);

		tmp->field2 = gtk_entry_new_with_max_length(128);
		gtk_widget_set_usize(tmp->field2,35,-1);
		ADD_TO_TABLE2(tmp->field2,1);
	}
	vPos++;

	ADD_TO_TABLE2(gtk_label_new("Policy"),0);

	GtkWidget *menu  = gtk_menu_new();
	GtkWidget *allow = gtk_menu_item_new_with_label("ALLOW");
        gtk_signal_connect_object(GTK_OBJECT(allow), "activate",
                                  GTK_SIGNAL_FUNC(setPolicyAllow),
                                  (GtkObject *) tmp);
	GtkWidget *deny	 = gtk_menu_item_new_with_label("DENY");
        gtk_signal_connect_object(GTK_OBJECT(deny), "activate",
                                  GTK_SIGNAL_FUNC(setPolicyDeny),
                                  (GtkObject *) tmp);

	gtk_menu_append((GtkMenu *) menu,allow);
	gtk_menu_append((GtkMenu *) menu,deny);

	GtkWidget *option = gtk_option_menu_new();
	gtk_option_menu_set_menu((GtkOptionMenu *) option,menu);
	ADD_TO_TABLE3(option,1,4);
	vPos++;

	/*
	 *	Fill'r in
	 */
	tmp->allow = TRUE;

	/*------------------------------------------------------*
	 *			Button portion			*
	 *------------------------------------------------------*/

	/*
	 *	Apply button
	 */
	GtkWidget *done_button = gtk_button_new_with_label("Apply");
        gtk_signal_connect_object(GTK_OBJECT (done_button), "clicked",
                                  GTK_SIGNAL_FUNC (gottaRuleApply),
                                  (GtkObject *) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  done_button);

	/*
	 *	Cancel button
	 */
	GtkWidget *cancel_button = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
			   GTK_SIGNAL_FUNC(gottaRuleCancel),(gpointer) tmp);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area),
			  cancel_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all(dialog);
}

