/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

/*
 *	hooverSortCompare() is a derivative of gtkclist.c:default_compare()
 *	that adds the ability to sort on the numeric value in the cell.
 */

gint
hooverSortCompare(GtkCList *clist,gconstpointer ptr1,gconstpointer ptr2)
{
	char	*text1 = NULL;
	char	*text2 = NULL;

	GtkCListRow	*row1 = (GtkCListRow *) ptr1;
	GtkCListRow	*row2 = (GtkCListRow *) ptr2;

	switch (row1->cell[clist->sort_column].type) {
	case GTK_CELL_TEXT:
		text1 = GTK_CELL_TEXT(row1->cell[clist->sort_column])->text;
		break;
	case GTK_CELL_PIXTEXT:
		text1 = GTK_CELL_PIXTEXT(row1->cell[clist->sort_column])->text;
		break;
	default:
		break;
	}

	switch (row2->cell[clist->sort_column].type) {
	case GTK_CELL_TEXT:
		text2 = GTK_CELL_TEXT(row2->cell[clist->sort_column])->text;
		break;
	case GTK_CELL_PIXTEXT:
		text2 = GTK_CELL_PIXTEXT(row2->cell[clist->sort_column])->text;
		break;
	default:
		break;
	}

	if (!text2)
		return(text1 != NULL);
	if (!text1)
		return(-1);

	/*
	 *	NOTE: This is where this compare varies from the
	 *	standard compare function. It converts things 
	 *	that look like number strings into their integer 
	 *	values for the comparisons.
	 */

	if (isdigit(*text1) && isdigit(*text2)) {
		int	num1 = atoi(text1);
		int	num2 = atoi(text2);
		return(num1 - num2);
	}
	return(strcmp(text1, text2));
}

void
hooverSortOnColumn(GtkCList * /*clist*/,gint col,gpointer user_data)
{
	int	prev_col;
	GtkCList	*theList = (GtkCList *) user_data;

	prev_col = (int) gtk_object_get_data(GTK_OBJECT(theList),
						"sort_column");
	if (prev_col == col) {
		// re-selected col, should switch the direction
	}
	gtk_clist_set_sort_column(theList,col);
	gtk_object_set_data(GTK_OBJECT(theList),"sort_column",(gpointer) col);
	gtk_clist_sort(theList);
}
