/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     This is the EMP node specific code.
 *
 */

#include "hoover.h"
#include "node.h"
#include "EMPservices.h"
#include "libgessie/Utils.h"

static void
handleNodeConsoleInquiry(InputEntry *entry,gpointer clientData)
{
	Node *node = (Node *) clientData;

	(void) strtok(entry->string,":");	// skip INQUIRY
	(void) strtok(NULL,":");		// node id
	char *device = strtok(NULL,":");	// device
//	char *login = strtok(NULL,":");		// login
//	char *passwd = strtok(NULL,":");	// password

	node->console = strdup(device ? device : "");
}

void 
EMPservices::handleInquiryMessage(InputEntry *entry)
{
	(void) strtok(entry->string,":");	// skip NODELIST
	(void) strtok(NULL,":");		// the node id
	char *sernum = strtok(NULL,":");	// the serial number
	char *empprt = strtok(NULL,":");	// the EMP port
	(void) strtok(NULL,":");		// the state of the node
	char *passwd = strtok(NULL,":");	// password

	/*
	 *	NOTE: should check node_id to make sure
	 */

	parent->serial_number = sernum ? strdup(sernum) : NULL;
	parent->emp_port      = empprt ? strdup(empprt) : NULL;
	parent->password      = passwd ? strdup(passwd) : NULL;

	/*
	 *	Do a serial console inquiry.
	 */
	doConsoleInquiry(handleNodeConsoleInquiry);
}

void 
EMPservices::handleInventoryMessage(InputEntry *entry)
{
	GtkWidget	*item;
	char	buffer[1024];

	/*
	 *	board fields
	 */
	(void) strtok(entry->string,":");	// skip NODEINVINFO
	char    *bmanuf     = strtok(NULL,":");
	char	*bprodnm    = strtok(NULL,":");
	(void) strtok(NULL,":");	/* char	*bpartnum   */
 	(void) strtok(NULL,":");	/* char	*brdserial  */
	char	*beth0addr  = strtok(NULL,":");
	/*
	 *	product fields
	 */
#ifdef	notdef
	char	*pmanuf     = strtok(NULL,":");
	char	*pprodnm    = strtok(NULL,":");
	char	*ppartnum   = strtok(NULL,":");
	char	*pprodver   = strtok(NULL,":");
	char	*pprodser   = strtok(NULL,":");
	char	*pprodasset = strtok(NULL,":");
#endif	/*notdef*/

	if (beth0addr == NULL) {
		parent->ip = NULL;
	}
	else if ((parent->ip == NULL) || (*parent->ip == '\0')) {
		parent->ip = strdup(beth0addr);
	}
	parent->clearSubtree();	// clear out items from before, if any
	/*
	 *	eth0 address
	 */

	sprintf(buffer,"eth0  = %s",parent->ip);
	item = gtk_tree_item_new_with_label(buffer);
	NONSELECT(item);
	parent->addToSubtree(item);
	gtk_widget_show(item);

	/*
	 *	board information
	 */
	if ((bmanuf != NULL) && (bprodnm != NULL)) {
		sprintf(buffer,"board = %s %s",bmanuf,bprodnm);
		item = gtk_tree_item_new_with_label(buffer);
		NONSELECT(item);
		parent->addToSubtree(item);
		gtk_widget_show(item);
	}

	parent->collapseSubtree();
}
