/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:	Dean Johnson (dtj@sgi.com)
 *
 * Purpose:	The HooverControl panel is the top level panel for
 *		hoover that contains two stacked panels. It opens
 *		a MainListPanel in the top panel and an AboutPanel
 *		in the lower panel.
 *
 */

#include "libgessie/PanelContainer.h"
#include "HooverControl.h"
#include "MainListPanel.h"
#include "AboutPanel.h"

#define	ADD_PANE_TOP(a,b)	gtk_paned_add1((GtkPaned *) a,(b))
#define	ADD_PANE_BOT(a,b)	gtk_paned_add2((GtkPaned *) a,(b))

HooverControl::HooverControl(char * /*str*/)
{
	GtkWidget	*topPane;
	GtkWidget	*pane;

        /*------------------------------------------------------*
         *                   TOPLEVEL NOTEBOOK                  *
         *------------------------------------------------------*/

	theBaseWidget = gtk_table_new(2 /*rows*/,1 /*cols*/,FALSE);
	gtk_widget_show(theBaseWidget);

                /*
                 *              PANE
                 */

	topPane = gtk_vpaned_new();
	gtk_widget_show(topPane);
	gtk_table_attach(GTK_TABLE(theBaseWidget),topPane,0,1,1,2,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),0,0);

	pane = gtk_vpaned_new();
	gtk_widget_show(pane);
	ADD_PANE_TOP(topPane,pane);
	
	topPC = new PanelContainer();
	topPC->setParent(this);
	ADD_PANE_TOP(pane,topPC->baseWidget());

	/*
	 *
	 */

	mainList = new MainListPanel(topPC);
	mainList->setSizes(300,150);
	topPC->add(mainList,"VACM List");

	botPC = new PanelContainer();
	botPC->setParent(this);
	ADD_PANE_BOT(pane,botPC->baseWidget());

	AboutPanel *aboutPanel = new AboutPanel();
//	aboutPanel->setSizes(200,150);
	botPC->add(aboutPanel,"About Hoover");

	aboutPanel->setSelfDestruct();
}

void
HooverControl::doMenu(GtkWidget *parent)
{
        Panel::doMenu(parent);          // put standard pieces in menu
        /*--------------------------*/
}
