/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */
/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     Handles the System Event Log Panel.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <time.h>
#include <gtk/gtk.h>
#include "SELListPanel.h"
#include "node.h"
#include "input_queue.h"
#include "connection.h"
#include "libgessie/Gessie.h"
#include "dialogs.h"
#include "node_list.h"
#include <assert.h>
#include "hoover.h"
#include "ClistHacks.h"

static void	handleNodeDownloadLog(InputEntry *entry,gpointer clientData);

struct sensortype {
	unsigned char type;
	char	*name;
};

#ifdef	notdef
static struct sensortype sensortypelist[] = {
  { 0x01, "Temperature"		},
  { 0x02, "Voltage"		},
  { 0x03, "Current"		},
  { 0x04, "Fan"			},
  { 0x05, "Physical Security"	},
  { 0x06, "Platform Security"	},
  { 0x07, "Processor"		},
  { 0x08, "Power Supply"	},
  { 0x09, "Power Unit"		},
  { 0x0a, "Cooling Device"	},
  { 0x0b, "Other Sensor"	},
  { 0x0c, "Memory"		},
  { 0x0d, "Drive Slot (Bay)"	},
  { 0x0e, "POST Memory Resize"	},
  { 0x0f, "POST Error"		},
  { 0x10, "Event Logging Disabled" },
  { 0x11, "Watchdog 1"		},
  { 0x12, "System Event"	},
  { 0x13, "Critical Interrupt"	},
  { 0x14, "Button"		},
  { 0x15, "Module / Board"	},
  { 0x16, "Microcontroller / Coprocessor" },
  { 0x17, "Add-in Card"		},
  { 0x18, "Chasis"		},
  { 0x19, "Chipset"		},
  { 0x1a, "Other FRU"		},
  { 0x1b, "Cable / Interconnect" },
  { 0x1c, "Terminator"		},
  { 0x1d, "System Boot"		},
  { 0x1e, "Boot Error"		},
  { 0x1f, "O/S Boot"		},
  { 0x20, "O/S Critical Stop"	},
  { 0x21, "Slot / Connector"	},
  { 0x22, "System ACPI Power State" },
  { 0x00, 0			}
};
#endif	/*notdef*/

static struct clist_column_info column_data[] = {
{ "Rec. ID", 		    50,FALSE,	GTK_JUSTIFY_LEFT },	/* Rec ID */
{ "Type", 		   120,TRUE,	GTK_JUSTIFY_LEFT },	/* type   */
{ "Time Stamp", 	   110,TRUE,	GTK_JUSTIFY_LEFT },	/* stamp  */
{ "Generator ID", 	    80,TRUE,	GTK_JUSTIFY_LEFT },	/* genid  */
{ "Emv Rev", 		    50,FALSE,	GTK_JUSTIFY_LEFT },	/* rev    */
{ "Sensor Type and Number",140,TRUE,	GTK_JUSTIFY_LEFT },	/* sensor */
{ "Event Description", 	   250,TRUE,	GTK_JUSTIFY_LEFT },	/* desc   */
};


SELListPanel::SELListPanel(PanelContainer * /*p*/, Node *node,EventLogType evl)
{
	theNode = node;
	logType = evl;
        insertPoint = 0;

//	setParent(p);
	theBaseWidget = gtk_table_new(2 /*rows*/,1 /*cols*/,FALSE);
	show();

	GtkWidget *scrollw = gtk_scrolled_window_new((GtkAdjustment *) NULL,
						     (GtkAdjustment *) NULL);

	gtk_container_set_border_width(GTK_CONTAINER(scrollw), 5);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollw),
				       GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);
	gtk_widget_show(scrollw);
	assert(scrollw != NULL);

        gtk_table_attach(GTK_TABLE(theBaseWidget),scrollw,0,1,0,1,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND), 
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),0,0);

	if (logType == EV_LOG_TEXT) {
		/*
		 *	Simple Text File
		 */
		theContentWidget = (GtkWidget *) gtk_text_new(
							(GtkAdjustment *) NULL,
                                                      	(GtkAdjustment *) NULL);
		gtk_widget_show(theContentWidget);
		gtk_object_set_user_data(GTK_OBJECT(theContentWidget),this);
		gtk_container_add(GTK_CONTAINER(scrollw), theContentWidget);
		gtk_widget_show(theContentWidget);
	}
	else	{
		/*
		 *	System Event Log Matrix
		 */
		
		int cnt = 7;
		int	i;
		gchar   **clist_titles;

		clist_titles = (gchar **) malloc(sizeof(gchar *) * cnt);
		for (i = 0; i < 7; i++) {
			clist_titles[i] = column_data[i].title;
		}

		/*------------------------------*
		 *		CLIST		*
		 *------------------------------*/

		theContentWidget = gtk_clist_new_with_titles(cnt,clist_titles);
		free(clist_titles);
		gtk_clist_set_selection_mode(GTK_CLIST(theContentWidget), 
					     GTK_SELECTION_SINGLE);
		gtk_signal_connect(GTK_OBJECT(theContentWidget), "click-column",
				   GTK_SIGNAL_FUNC(hooverSortOnColumn), 
				   (gpointer) theContentWidget);
		gtk_clist_set_compare_func(GTK_CLIST(theContentWidget),
					   hooverSortCompare);
		gtk_object_set_data(GTK_OBJECT(theContentWidget),"sort_column",
				    (void *) BOGUS_SORT_COLUMN);
		gtk_container_add(GTK_CONTAINER(scrollw), theContentWidget);
		gtk_widget_show(theContentWidget);
		gtk_clist_set_shadow_type(GTK_CLIST(theContentWidget),
					  GTK_SHADOW_OUT);
		for (i = 0; i < cnt; i++) {
			gtk_clist_set_column_width(GTK_CLIST(theContentWidget),
						   i,column_data[i].width);
			gtk_clist_set_column_justification(GTK_CLIST(theContentWidget),
						i,column_data[i].justification);
			gtk_clist_set_column_visibility(
						GTK_CLIST(theContentWidget),i,
						column_data[i].visible);
		}
	}
	activateLogging();
}

void
SELListPanel::activateLogging()
{
	static int	beenHereBefore = 0;

	/*--------------------------------------------------------------*
	 *			Get the log				*
	 *--------------------------------------------------------------*/

	if (globalDemoMode && !beenHereBefore) {
		beenHereBefore = 1;
		/*
		 *	because we are in debug mode, none of the nodes are
		 *	really there, thus we get some errors, but we need
		 *	the info. So set up the error jobnum to go through
		 *	handleNodeDownloadLog().
		 */
		newJobEntry(0,FALSE,handleNodeDownloadLog,(gpointer) this);
	}
	theNode->services->doSELLogging((gpointer) this,handleNodeDownloadLog);
}

int
SELListPanel::interested(char * /*msgtype*/,gpointer /*data*/)
{
	return(0);
}

void
SELListPanel::doMenu(GtkWidget *parent)
{
        Panel::doMenu(parent);          // put standard pieces in menu
        /*--------------------------*/
}

static void
handleNodeDownloadLog(InputEntry *entry,gpointer clientData)
{
	char	*selrec[7];
	char	*service;
	InputEntry *tmp;
	SELListPanel	*sel = (SELListPanel *) clientData;
	char	buf[128];

//	printInputEntries(entry,"SEL Entry:");
	sel->freeze();
	for (tmp = entry; tmp != NULL; tmp = tmp->next) {
		if (tmp->string == NULL) continue;
		if (sel->logType == EV_LOG_TEXT) {
			if (!strncasecmp(tmp->string,"DOWNLOAD_LOG:",13)) {
				sel->appendText((const char *) &tmp->string[13]);
			}
			else	{
				sel->appendText((const char *) tmp->string);
			}
			sel->appendText("\n");
		}
		else	{
			service = strtok(tmp->string,":");
			(void) strtok(NULL,":");	// node serial number

			selrec[0] = strtok(NULL,":");	// record id
			selrec[1] = strtok(NULL,":");	// record type
			selrec[2] = strtok(NULL,":");	// record timestamp
			selrec[3] = strtok(NULL,":");	// gen id
			selrec[4] = strtok(NULL,":");	// record fmt version

			selrec[5] = strtok(NULL,":");	// sensor type
			if (selrec[5] == NULL) continue;
			if (selrec[2] != NULL) {
				unsigned long int uli = strtoul(selrec[2],NULL,0);
				if (uli==4 && !strcmp(selrec[5],"Physical Security")) {
					/*
					 *	This is a bogus spurious SEL msg
					 *	that needs to be weeded out.
					 */
					continue;
				}
				struct tm *lcl = localtime((const time_t *) &uli);
				strftime(buf,128,"%D %k:%M:%S",lcl);
				selrec[2] = buf;
			}
#ifdef	notdef
			/*
			 *	The sensor type should be passed as a number
			 *	but seems to be passed as a string.
			 */
			unsigned char senType = (unsigned char) atoi(selrec[5]);
			for (i = 0; sensortypelist[i].name; i++) {
				if (sensortypelist[i].type == senType) {
					selrec[5] = sensortypelist[i].name;
					break;
				}
			}
#endif	/*notdef*/

			(void) strtok(NULL,":");	// sensor name
			selrec[6] = strtok(NULL,":");	// event description

			gtk_clist_append(GTK_CLIST(sel->contentWidget()), 
					 selrec);
		}
	}
	sel->thaw();
}
