/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     Handles the Sensor Data Panel.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <gtk/gtk.h>
#include "SENSListPanel.h"
#include "node.h"
#include "input_queue.h"
#include "libgessie/Gessie.h"
#include <assert.h>
#include "hoover.h"
#include "ClistHacks.h"

#define	SENSOR_FAST	 15		//	every 15 seconds
#define	SENSOR_MED	 30		//	every 30 seconds
#define	SENSOR_SLOW	 60		//	every 60 seconds
#define	SENSOR_SETBACK	300		//	rechecking if sensors active

class	RowSensorMapping {
private:
	int	row;
	int	sensor;
public:
	RowSensorMapping(int s,int r) { 
		row = r;
		sensor = s;
	}
	int	getRow()		{ return(row);		}
	int	getSensor()		{ return(sensor);	}
};

static int	checkActualSensors(gpointer data);
static void	handleNodeGetSensor(InputEntry *entry,gpointer clientData);

static struct clist_column_info column_data[] = {
{ "Sensor #",	 50,TRUE,	GTK_JUSTIFY_LEFT	},
{ "Name",	140,TRUE,	GTK_JUSTIFY_LEFT	},
{ "Type",	 80,TRUE,	GTK_JUSTIFY_LEFT	},
{ "Actual",	 50,TRUE,	GTK_JUSTIFY_RIGHT	},
{ "Min",	 50,TRUE,	GTK_JUSTIFY_RIGHT	},
{ "Max",	 50,TRUE,	GTK_JUSTIFY_RIGHT	},
{ "",		 50,FALSE,	GTK_JUSTIFY_LEFT	},
{ "",		 80,FALSE,	GTK_JUSTIFY_LEFT	},
{ "",		 80,FALSE,	GTK_JUSTIFY_LEFT	},
{ "",		 80,FALSE,	GTK_JUSTIFY_LEFT	},
{ "",		 80,FALSE,	GTK_JUSTIFY_LEFT	},
};

SENSListPanel::SENSListPanel(PanelContainer * /*p*/, Node *node)
{
	int cnt = 7 /*11*/;
	int	i;
	gchar   **clist_titles/* = column_titles*/;

	clist_titles = (gchar **) malloc(sizeof(gchar *) * cnt);
	for (i = 0; i < cnt /*11*/; i++) {
		clist_titles[i] = column_data[i].title;
	}

	sensorRate	= SENSOR_MED;
	savedRate	= SENSOR_MED;
	timeout_handler	= 0;
	theMapping	= NULL;
	theNode		= node;

//	setParent(p);
	theBaseWidget = gtk_table_new(2 /*rows*/,1 /*cols*/,FALSE);
	show();

	GtkWidget *scrollw = gtk_scrolled_window_new((GtkAdjustment *) NULL,
						     (GtkAdjustment *) NULL);

	gtk_container_set_border_width(GTK_CONTAINER(scrollw), 10);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrollw),
				       GTK_POLICY_AUTOMATIC,GTK_POLICY_ALWAYS);
	gtk_widget_show(scrollw);
	assert(scrollw != NULL);

        gtk_table_attach(GTK_TABLE(theBaseWidget),scrollw,0,1,0,1,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND), 
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),0,0);

		/*------------------------------*
		 *		CLIST		*
		 *------------------------------*/

	theContentWidget = gtk_clist_new_with_titles(cnt,clist_titles);
	gtk_clist_set_selection_mode(GTK_CLIST(theContentWidget), 
				     GTK_SELECTION_SINGLE);
	gtk_signal_connect(GTK_OBJECT(theContentWidget), "click-column",
			   GTK_SIGNAL_FUNC(hooverSortOnColumn),
			   (gpointer) theContentWidget);
	gtk_clist_set_compare_func(GTK_CLIST(theContentWidget),hooverSortCompare);
        gtk_clist_set_sort_column(GTK_CLIST(theContentWidget),0);
        gtk_object_set_data(GTK_OBJECT(theContentWidget),"sort_column",
                            (void *) 0);

	gtk_widget_show(theContentWidget);
	gtk_clist_set_shadow_type(GTK_CLIST(theContentWidget),GTK_SHADOW_OUT);
	for (i = 0; i < cnt; i++) {
		gtk_clist_set_column_width(GTK_CLIST(theContentWidget),i,
					   column_data[i].width);
		gtk_clist_set_column_justification(GTK_CLIST(theContentWidget),
					   i,column_data[i].justification);
	}
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scrollw),
					      theContentWidget);
	activateLogging();
}

SENSListPanel::~SENSListPanel()
{
	/* should nuke timeout_handler if still around */
}

void
SENSListPanel::activateLogging()
{
	theNode->services->doSENLogging((gpointer) this,handleNodeGetSensor);
}

int
SENSListPanel::interested(char * /*msgtype*/,gpointer /*data*/)
{
	return(0);
}

void
SENSListPanel::setTimer(int rate)
{
	if (rate == SENSOR_SETBACK && sensorRate != SENSOR_SETBACK) {
		savedRate = sensorRate;	// save old rate
	}
	sensorRate = rate;
	if (timeout_handler != 0)
		gtk_timeout_remove(timeout_handler);
	timeout_handler = gtk_timeout_add((sensorRate*1000),checkActualSensors,
					  (gpointer) this);
}

void
setRefreshRate(gpointer callback_data,GtkWidget *w)
{
	SENSListPanel *sel = (SENSListPanel *) callback_data;
	int	rate = (int) gtk_object_get_data(GTK_OBJECT(w),"info");

	sel->setTimer(rate);
}

void
SENSListPanel::doMenu(GtkWidget *parent)
{
	GtkWidget	*menu_item;
	GSList	*group = NULL;

        Panel::doMenu(parent);          // put standard pieces in menu
        /*--------------------------*/

#define PANEL_ADD_MENU_RADIO(str,cb,cdata,data)				\
                menu_item = gtk_radio_menu_item_new_with_label(group,str); \
		gtk_object_set_data(GTK_OBJECT(menu_item),"info", 	\
				    (void *) data); 			\
		if (sensorRate == data) {				\
			gtk_check_menu_item_set_active(			\
				GTK_CHECK_MENU_ITEM(menu_item),TRUE);	\
		}							\
                gtk_signal_connect_object(GTK_OBJECT(menu_item),        \
                                          "activate",GTK_SIGNAL_FUNC(cb),\
                                          (GtkObject *) cdata);         \
		gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(\
						menu_item),TRUE);   	\
                gtk_menu_append(GTK_MENU(parent), menu_item);           \
                gtk_widget_show(menu_item)

        PANEL_ADD_MENU_RADIO("Slow Refresh",setRefreshRate,this,SENSOR_SLOW);
        PANEL_ADD_MENU_RADIO("Medium Refresh",setRefreshRate,this,SENSOR_MED);
        PANEL_ADD_MENU_RADIO("Fast Refresh",setRefreshRate,this,SENSOR_FAST);
}

static void
handleSensorValue(InputEntry *entry,gpointer clientData)
{
	SENSListPanel	*sel = (SENSListPanel *) clientData;
	int	row;

	for (InputEntry *tmp = entry; tmp != NULL; tmp = tmp->next) {
                char *response = strtok(tmp->string,":"); // "SENSOR"?
		if (!strcmp(response,"JOB_ERROR")) {
			/*
			 *	Houston, we have a problem!
			 */
			char *info = strtok(NULL,":");
			if (!strcmp(info,"Transport endpoint is not connected")) {
				// the daemon on the other end is toast
				sel->setTimer(SENSOR_SETBACK);
				sel->removeActualSensorData();
			}
			return;
		}

		/*
		 *	If we are coming off a 'sensors aint available'
		 *	setback, set the timer back to where it was when
		 *	it was setback.
		 */
		if (sel->sensorRate == SENSOR_SETBACK) {
			sel->setTimer(sel->savedRate);
		}

		/*
		 *	Everything should be okay
		 */
		char *sensor_num = strtok(NULL,":");
		char *value	 = strtok(NULL,":");
		row = gtk_clist_find_row_from_data(GTK_CLIST(sel->contentWidget()),
						   (gpointer) atoi(sensor_num));
		if (row < 0) continue;
		gtk_clist_set_text(GTK_CLIST(sel->contentWidget()),row,3,value);
	}
}

static void
getActualData(gpointer mapdata,gpointer data)
{
	SENSListPanel *panel = (SENSListPanel *) data;
	RowSensorMapping *tmp = (RowSensorMapping *) mapdata;

	panel->theNode->services->doGetSensorValue((gpointer *) panel,
					 	   handleSensorValue,
					 	   tmp->getSensor());
}

static int
checkActualSensors(gpointer data)
{
	SENSListPanel *panel = (SENSListPanel *) data;

	g_list_foreach((GList *) panel->theMapping,getActualData,
		       (gpointer) panel);
	return(1);
}

static void
nukeActualData(gpointer mapdata,gpointer data)
{
	SENSListPanel *panel = (SENSListPanel *) data;
	RowSensorMapping *tmp = (RowSensorMapping *) mapdata;

	gtk_clist_set_text(GTK_CLIST(panel->contentWidget()),tmp->getRow(),
			   3,"???");
}

void
SENSListPanel::removeActualSensorData()
{
	g_list_foreach((GList *) theMapping,nukeActualData,(gpointer) this);
}

static void
handleSensorThreshold(InputEntry *entry,gpointer clientData)
{
	SENSListPanel	*sel = (SENSListPanel *) clientData;
	Node	*node = sel->theNode;
	char	*minVal;
	char	*lowerNonCrit;
	char	*lowerNonRecov;
	char	*lowerCritical;
	char	*maxVal;
	char	*upperNonCrit;
	char	*upperNonRecov;
	char	*upperCritical;
	char	*number;
	int	row;

	for (InputEntry *tmp = entry; tmp != NULL; tmp = tmp->next) {
                (void) strtok(tmp->string,":"); // "THRESHOLD"
		number		= strtok(NULL,":");
		minVal		= strtok(NULL,":");
		lowerNonCrit	= strtok(NULL,":");
		lowerNonRecov	= strtok(NULL,":");
		lowerCritical	= strtok(NULL,":");
		upperNonCrit	= strtok(NULL,":");
		upperNonRecov	= strtok(NULL,":");
		upperCritical	= strtok(NULL,":");
		maxVal		= strtok(NULL,":");

		row = gtk_clist_find_row_from_data(GTK_CLIST(sel->contentWidget()),
						   (gpointer) atoi(number));
		if (row < 0) continue;

		gtk_clist_set_text(GTK_CLIST(sel->contentWidget()),row,3,"???");
		gtk_clist_set_text(GTK_CLIST(sel->contentWidget()),row,4,
				   minVal);
		gtk_clist_set_text(GTK_CLIST(sel->contentWidget()),row,5,
				   maxVal);
		node->services->doGetSensorValue((gpointer *) sel,
						 handleSensorValue,
						 atoi(number));
	}

}

static void
handleNodeGetSensor(InputEntry *entry,gpointer clientData)
{
	char	*selrec[11];
	SENSListPanel	*sel = (SENSListPanel *) clientData;
	Node	*node = sel->theNode;
	int	row;

//	printInputEntries(entry,"SENS Entry:");
	gtk_clist_freeze(GTK_CLIST(sel->contentWidget()));
	for (InputEntry *tmp = entry; tmp != NULL; tmp = tmp->next) {
		(void) strtok(tmp->string,":");	// "SENSOR"
		selrec[0] = strtok(NULL,":");	// sensor number
		selrec[1] = strtok(NULL,":");	// sensor name
		selrec[2] = strtok(NULL,":");	// sensor type

		selrec[3] = "";
		selrec[4] = "";
		selrec[5] = "";
		selrec[6] = "";
		selrec[7] = "";
		selrec[8] = "";
		selrec[9] = "";
		selrec[10] = "";
		row = gtk_clist_append(GTK_CLIST(sel->contentWidget()),selrec);
		int	sensor_num = atoi(selrec[0]);
 		gtk_clist_set_row_data(GTK_CLIST(sel->contentWidget()),row,
				       (gpointer) sensor_num);
		sel->theMapping = g_slist_append(sel->theMapping,
		       (gpointer)(new RowSensorMapping(sensor_num,row)));
		/*
		 *	Now get the thresholds for the sensor.
		 */
		node->services->getSENThresholds(selrec[0],(gpointer) sel,
				       		 handleSensorThreshold);
	}
        gtk_clist_sort(GTK_CLIST(sel->contentWidget()));
	gtk_clist_thaw(GTK_CLIST(sel->contentWidget()));
	/*
	 *	Now light off the timer to update the actual values.
	 */
	checkActualSensors((gpointer) sel);
	sel->timeout_handler = gtk_timeout_add((sel->sensorRate*1000),
					       checkActualSensors,
					       (gpointer) sel);
}
