/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 *      Herein lie things that should probably be in gtk+, or are working
 *      around some particular bit of "character" in gtk.
 */

#include <gtk/gtk.h>

GtkWidget *
gtk_tree_item_get_label(GtkWidget *w)
{
	GList *children = gtk_container_children(GTK_CONTAINER(w));
	GtkWidget *child = (GtkWidget *)(children ? children->data : NULL);
	g_list_free (children);
	return(child);
}

GtkWidget *
gtk_tree_item_get_clist(GtkWidget *w)
{
	GList *children = gtk_container_children(GTK_CONTAINER(w));
	GtkWidget *child = (GtkWidget *) (children ? children->data : NULL);
	g_list_free(children);
	return(child);
}

GtkWidget*
gtk_tree_item_new_with_clist(int cols)
{
	GtkWidget *tree_item;
	GtkWidget *clist_widget;

	tree_item = gtk_tree_item_new();
	clist_widget = gtk_clist_new (cols);
	gtk_container_add(GTK_CONTAINER(tree_item), clist_widget);
	gtk_widget_show(clist_widget);
	return(tree_item);
}

/*----------------------------------------------------------------------*
 *				STYLE STUFF				*
 *----------------------------------------------------------------------*/

GtkStyle *
gtk_create_reverse_style()
{
	GtkStyle *style;

	style = gtk_style_copy(gtk_widget_get_default_style());
	style->fg[GTK_STATE_ACTIVE]   = style->white;
	style->text[GTK_STATE_ACTIVE] = style->white;
	style->bg[GTK_STATE_ACTIVE]   = style->black;
	return(style);
}

void
gtk_set_style_recursively(GtkWidget *widget,gpointer data)
{
	GtkStyle *style = (GtkStyle *) data;

	gtk_widget_set_style(widget,style);
	if (GTK_IS_CONTAINER(widget)) {
		gtk_container_foreach(GTK_CONTAINER(widget),
				      gtk_set_style_recursively,style);
	}
}
