/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:	Dean Johnson (dtj@sgi.com)
 *
 * Purpose:	This is the main program of Hoover.
 *
 * Notes:	Lots of stuff gets dumped in here until it gets moved
 *		to a more appropriate place.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <gtk/gtkenums.h>
#include <glib.h>
#ifdef	notdef
#include <gnome.h>
#endif	/*notdef*/
#include <assert.h>
#include "node.h"
#include "input_queue.h"
#include "dialogs.h"
#include "node_list.h"
#include "MainListPanel.h"
#include "ConsolePanel.h"
#include "HooverControl.h"
#include "HooverMessages.h"
#include "libgessie/Gessie.h"
#define	MAIN
#include "libgessie/Globals.h"
#include "libgessie/Broker.h"
#include "hoover.h"
#include "Plugins.h"

#define	DEFAULT_RECHECK_INTERVAL	1.0

#define DEFAULT_SCAN_MASK       "192.168.0.0/24"

static float	recheck_interval = DEFAULT_RECHECK_INTERVAL;

static void	nexxusScan(gpointer,guint,GtkWidget *);
static void	refreshNodelistCallback(gpointer,guint,GtkWidget *);
static void	setBootingState(gpointer node,gpointer data);

extern void	checkClusterPropertyStates(Node *);
extern void	doTheClusterAdd(char *hostname,gpointer clientData = NULL,
				gpointer callData = NULL);
extern void	doNexxusScan(char *text, gpointer clientData,gpointer callData);
static void	iterativeConnect();

GtkItemFactoryEntry HooverMenuItems[] = {

		/*--------------------------------------*
		 *		FILE MENU		*
		 *--------------------------------------*/

       { "/_File",                              (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      "<Branch>"              },
       { "/File/_Open",                         "<control>O",
         (GtkItemFactoryCallback) NULL, 0,      (gchar *) NULL          },
       { "/File/sep0",                          (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      "<Separator>"           },
       { "/File/Quit",                          "<control>Q",
         (GtkItemFactoryCallback) gtk_main_quit, 0,                      (gchar *) NULL          },

		/*--------------------------------------*
		 *		OPTIONS MENU		*
		 *--------------------------------------*/

       { "/_Options",                           (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      "<Branch>"              },
#ifdef	notdef
       { "/Options/Disable Tooltip Popups",     (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      "<CheckItem>"           },
       { "/Options/sep1",                       (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      "<Separator>"           },
#endif	/*notdef*/
       { "/Options/Scan for Nexxus",            "<control>s",
         (GtkItemFactoryCallback) nexxusScan, 0,(gchar *) NULL 		},
       { "/Options/Refresh Nodelists",		"<control>r",
         (GtkItemFactoryCallback) refreshNodelistCallback, 0,(gchar *) NULL },

		/*--------------------------------------*
		 *		HELP MENU		*
		 *--------------------------------------*/

       { "/_Help",                              (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      "<LastBranch>"          },
       { "/_Help/About",                        (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      (gchar *) NULL		},
       { "/_Help/Main Help",                    (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      (gchar *) NULL		},
       { "/_Help/Latest News about Hoover",     (gchar *) NULL,
         (GtkItemFactoryCallback) NULL, 0,      (gchar *) NULL		}
     };

gint    nHooverMenuItems = sizeof(HooverMenuItems) / sizeof(HooverMenuItems[0]);

gboolean	globalDemoMode = FALSE;	// should we fake stuff?
int		globalVerboseMode = 0;	// should we holler at every chance?
gboolean	globalStatusChecksSuspended = FALSE;
static gboolean	noHostFlag = FALSE;

GSList		*theNodeList = NULL;
GtkWidget	*theRootTable = NULL;
GtkWidget		*theList = NULL;

MainListPanel	*mainList = NULL;
GSList          *thePendingHosts = NULL;

	/*--------------------------------------------------------------*
	 *								*
	 *--------------------------------------------------------------*/

static void
checkNodeStatus(gpointer node_data,gpointer data)
{
	Node *node = (Node *) node_data;
	
	if (node == NULL) return;
	if (node->connection == NULL) return;
	if (node->connection->handle == NULL) return;
	if (!node->singularTypeP()) return;
	node->checkStatus(data);
}

int 
checkTreeStates(gpointer data)
{
	hooverSetStatus("Checking clusters...");
	g_list_foreach((GList *) theNodeList,checkNodeStatus,data);
	hooverSetStatus("");
	return(1);
}

static void
refreshNodelistCallback(gpointer,guint,GtkWidget*)
{
	refreshNodeLists();
}

static int 
generateSpuriousSEL(gpointer)
{
	Node *tmp;

	for (unsigned int i = 0; i < g_slist_length(theNodeList); i++) {
		tmp = (Node *) g_list_nth_data((GList *) theNodeList,i);
		tmp->services->doStimulateSELEntry(i);
	}
	return(0);
}

static void
sigpipe_handler(int /*info*/)
{
	informational_dialog("Broken Pipe Occurred!");
}

static void
dealWithSELEvent(char *inPtr)
{
	(void) strtok(inPtr,":");		// skip "SEL:"
	char	*serial   = strtok(NULL,":");	// serial number
	(void) strtok(NULL,":");	/* char	*rid	  */
 	(void) strtok(NULL,":");	/* char	*rtype	  */
 	(void) strtok(NULL,":");	/* char	*rtstamp  */
 	(void) strtok(NULL,":");	/* char	*rgenid	  */
 	(void) strtok(NULL,":");	/* char	*rformver */
 	(void) strtok(NULL,":");	/* char	*senstype */
 	(void) strtok(NULL,":");	/* char	*sensname */
	char	*descript = strtok(NULL,":");

	if (!strcmp(descript,"OEM System Boot Event")) {
		g_list_foreach((GList *) theNodeList,setBootingState,serial);
	}
}

static void
dealWithSerconEvent(char *inPtr)
{
	(void) strtok(inPtr,":");		// skip "CB:"
	char	*nodename = strtok(NULL,":");	// node name
	char	*msg = nodename + strlen(nodename) + 1;

	GSList *plist = gsGetPanelList(mainList,"ConsolePanel");
	if (plist != NULL) {
		for (unsigned int i = 0; i < g_slist_length(plist); i++) {
			ConsolePanel *cp = (ConsolePanel *) g_list_nth_data((GList *) plist,i);
			if (!strcmp(cp->theNode->name,nodename)) {
				//      raise it and return;
				cp->writeStr(msg);
				return;
			}
		}
	}
}

static void
dealWithWALLEvent(gpointer handle,char *inPtr)
{
	char	buf[1024];
	char	nexxus[1024];

	(void) strtok(inPtr,":");		// skip "WALL:"
	char	*user = strtok(NULL,":");	// user name
	char	*from = strtok(NULL,":");	// node name
	char	*msg = strtok(NULL,":");	// the mesg

	ConnectInfo *con = handleToConnectInfo(handle);
	if (con) {
		sprintf(nexxus,"  \nto nexxus '%s'",con->hostname);
	}
	else {
		*nexxus = '\0';
	}
	sprintf(buf,"VACM msg from (%s@%s)%s:  \n\"%s\"  ",user,from,nexxus,msg);
	(void) informational_dialog(buf);
}

static void
handleMiscMessages(InputEntry *entry,gpointer /*clientData*/)
{
	/*
	 *	Handle the miscellanous messages the come down the
	 *	pipe and deal with them sanely.
	 */
	for (InputEntry *tmp = entry; tmp; tmp = tmp->next) {
		if (!strncmp(tmp->string,"SEL:",4)) {
			dealWithSELEvent(tmp->string);
		}
		else if (!strncmp(tmp->string,"WALL:",3)) {
			dealWithWALLEvent(tmp->handle,tmp->string);
		}
		else if (!strncmp(tmp->string,"CB:",3)) {
			dealWithSerconEvent(tmp->string);
		}
	}
}

static void
delete_event(GtkWidget*,GdkEvent*,gpointer)
{
        gtk_main_quit();
}

int 
main(int argc,char *argv[])
{
	int	i;
	int	ch;
	int	recheck_milli = 0;
	int	hyper_milli = 0;
	char	*thisHost;
        char    buf[128];

	signal(SIGPIPE, sigpipe_handler);

	gtk_init(&argc,&argv);
	gdk_init(&argc, &argv);
//	gnome_init("hoover","1.0a",argc,argv);
//	gdk_imlib_init();

	PIM_Initialize();
	PIM_RunPreXHook();
	
	gsSetBrokerToplevelName("HooverControl");
	gessieSetAppName("Hoover");
	gessieSetAppVersion("1.0");

        while ((ch = getopt(argc,argv,"di:nv")) != EOF) {
                switch (ch) {
                case 'd': globalDemoMode = TRUE;		break;
                case 'i': recheck_interval = atof(optarg);	break;
		case 'n': noHostFlag = TRUE;			break;
                case 'v': globalVerboseMode++;			break;
                }
        }
	recheck_milli = (int)(recheck_interval*60.0*1000.0);
	if (recheck_milli > 0) {
		hyper_milli = recheck_milli / 4;
	}
	else	{
		hyper_milli = 15 * 1000;		// minimum 15 seconds
	}
	for (i = optind; i < argc; i++) {
		thisHost = strdup(argv[i]);
		thePendingHosts = g_slist_append(thePendingHosts,(gpointer)thisHost);
        }
	if (g_slist_length(thePendingHosts) == 0 && !noHostFlag) {
		char 	hostBuf[512];
		/*
		 *	No arguments were given, so use 'localhost'
		 */
		if (!gethostname(hostBuf,512)) {
			thisHost = strdup(hostBuf);
		}
		else	{
			thisHost = strdup("localhost");
		}
		thePendingHosts = g_slist_append(thePendingHosts,(gpointer)thisHost);
	}

	theRootWindow = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_widget_set_usize(theRootWindow,515,490);
        sprintf(buf,"%s %s",gessieGetAppName(),gessieGetAppVersion());
	gtk_window_set_title(GTK_WINDOW (theRootWindow),buf);
	gtk_signal_connect(GTK_OBJECT (theRootWindow), "delete_event",
			   GTK_SIGNAL_FUNC (delete_event), NULL);
	gtk_widget_show(theRootWindow);

        theRootTable = gtk_table_new(4 /*rows*/,1 /*cols*/,FALSE);
	gtk_widget_show(theRootTable);

	gtk_container_add((GtkContainer *) theRootWindow,theRootTable);

	/*--------------------------------------------------------------*
	 *			      Menu bar				*
	 *--------------------------------------------------------------*/

	theRootMenuBar = gsInitMainMenus(nHooverMenuItems,HooverMenuItems);
        gtk_table_attach(GTK_TABLE(theRootTable),theRootMenuBar,0,1,0,1,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 (enum GtkAttachOptions)(GTK_FILL),0,0);


	/*------------------------------------------------------*
	 *                   TOPLEVEL NOTEBOOK                  *
	 *------------------------------------------------------*/

	theRootNotebook = new PanelContainer();
	gtk_notebook_set_show_tabs(GTK_NOTEBOOK(theRootNotebook->baseWidget()),
				   FALSE);
	gtk_table_attach(GTK_TABLE(theRootTable),theRootNotebook->baseWidget(),
			 0,1,1,2,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),0,0);

	/*------------------------------------------------------*
	 *                   STATUS BAR                         *
	 *------------------------------------------------------*/

        GtkWidget *hbox = gtk_hbox_new(FALSE /*TRUE*/,3);
        gtk_widget_show(hbox);

	gtk_table_attach(GTK_TABLE(theRootTable),hbox,0,1,2,3,
			 (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 (enum GtkAttachOptions)(GTK_FILL),0,0);

	theRootProgressBar = gtk_progress_bar_new();
	gtk_widget_set_usize(theRootProgressBar,100,-1);
	gtk_box_pack_start((GtkBox *) hbox,theRootProgressBar,FALSE,TRUE,1);
	gtk_widget_show(theRootProgressBar);

	theRootStatusBar = gtk_statusbar_new();
	gtk_box_pack_start((GtkBox *) hbox,theRootStatusBar,TRUE,TRUE,1);
	gtk_widget_show(theRootStatusBar);

	/*--------------------------------------------------------------*
	 *			       Main list			*
	 *--------------------------------------------------------------*/

	HooverControl *mainPanel = new HooverControl("booga-booga");
	mainList = (MainListPanel *) mainPanel->theMainList();
	theList = mainList->contentWidget();

	theRootNotebook->add((Panel *) mainPanel,"VACM");

	/*--------------------------------------------------------------*
	 *			     DO THE DATA			*
	 *--------------------------------------------------------------*/

#ifdef	notdef
	for (i = 0; i < (int) g_slist_length(thePendingHosts); i++) {
		thisHost = (char *) g_list_nth_data((GList *) thePendingHosts,i);
		doTheClusterAdd(thisHost);
	}
#else	/*notdef*/
	iterativeConnect();
#endif	/*notdef*/

	/*--------------------------------------------------------------*
	 *	        Check on the node states every N minutes	*
	 *--------------------------------------------------------------*/

	checkTreeStates(NULL);	//	check it initially
	gtk_timeout_add(recheck_milli,checkTreeStates,(void *) CHECK_NORMALLY);
	gtk_timeout_add(hyper_milli,checkTreeStates,(void *) CHECK_HYPERACTIVE);
	if (globalDemoMode) {
		gtk_timeout_add((int)(recheck_interval*60.0*1000.0),
				generateSpuriousSEL, (void *) NULL);
	}

	/*--------------------------------------------------------------*
	 *--------------------------------------------------------------*/

	newJobEntry(0,FALSE,handleMiscMessages,(gpointer) NULL);

	/*
	 *	Let'r rip!
	 */
	PIM_RunPostXHook();
	gtk_main ();
	return(0);
}

/*----------------------------------------------------------------------*
 *----------------------------------------------------------------------*/

GtkWidget *
hooverGetStatusBar()
{
        return(theRootStatusBar);
}

void
hooverSetStatus(char *msg)
{
	static gboolean beenHere = 0;
	static guint	contextId;

	if (!beenHere) {
		contextId = gtk_statusbar_get_context_id((GtkStatusbar *) theRootStatusBar,"hoover");
		beenHere = TRUE;
	}
	gtk_statusbar_push((GtkStatusbar *) theRootStatusBar,contextId,(gchar *) msg);
	while (gtk_events_pending()) gtk_main_iteration();      // spin dry!
}

void
hooverClearStatus()
{
	hooverSetStatus("");
}

GtkWidget *
hooverGetProgressBar()
{
        return(theRootProgressBar);
}

void
hooverResetProgressBar()
{
        gtk_progress_set_format_string((GtkProgress *)theRootProgressBar,"");
        gtk_progress_bar_update((GtkProgressBar *)theRootProgressBar,0.0);
}

/*----------------------------------------------------------------------*
 *----------------------------------------------------------------------*/

static void
setBootingState(gpointer node_data,gpointer data)
{
	Node	*node   = (Node *) node_data;
	char	*serial = (char *) data;

	/*
	 *	If there is a serial number, compare it
	 */
	if (serial != NULL && strcmp(node->serial_number,serial)) return;
	node->setState(N_STATE_BOOTING);
	node->checkState = CHECK_HYPERACTIVE;	// watch for it to be done
	node->updateVisual();
}

static void
nexxusScan(gpointer,guint,GtkWidget *)
{
        get_textentry_dialog("Scan Network for Nexxus",doNexxusScan,
			     (gpointer) NULL,"Network  ","OK",
			     DEFAULT_SCAN_MASK);
}

/*----------------------------------------------------------------------*
 *----------------------------------------------------------------------*/

void
hooverNodeSelection(GtkWidget *item, gchar *signame)
{
	Node	*node = (Node *) gtk_object_get_data(GTK_OBJECT(item),"item");
	if (node == NULL) {
		gtk_item_deselect((GtkItem *) item);
		gsBroadcastBrokerMsg(mainList,HooverMessages.NodeUnselected,
				     (gpointer) node);
		return;
	}
	if (!strcmp(signame,"deselect")) {
		mainList->setCurrentSelection(NULL);
		gsBroadcastBrokerMsg(mainList,HooverMessages.NodeUnselected,
				     (gpointer) node);
	}
	else if (!strcmp(signame,"select")) {
		mainList->setCurrentSelection(item);
		gsBroadcastBrokerMsg(mainList,HooverMessages.NodeSelected,
				     (gpointer) node);
	}
}

/*----------------------------------------------------------------------*
 *		      Iterative Authentication Mechanism		*
 *----------------------------------------------------------------------*/

static void
connectToCluster(char *userid,char *password, char * /*password2*/,
		 gpointer clientData,gpointer /*callData*/)
{
	int	pos = (int) clientData;
	char	buf[1024];
	char	*host;
        Node    *node;

	/* 
	 *	connect up with the info 
	 */
	host = (char *) g_list_nth_data((GList *) thePendingHosts,pos);

        ConnectInfo	*tmp = new ConnectInfo(host);
        node = tmp->renderClusterNode(userid,password);
        theNodeList = g_slist_append(theNodeList, (gpointer) node);
        tmp->position = g_slist_length(theNodeList) - 1;

//	doTheClusterAdd(host);

	/* 
	 *	are we done? 
	 */
	pos++;
	if (pos >= (int) g_list_length((GList *) thePendingHosts)) return;

	/* 
	 *	now hook me up again 
	 */
	host = (char *) g_list_nth_data((GList *) thePendingHosts,pos);
	sprintf(buf,"Authentication Info for '%s'",host);
	get_passwordentry_dialog(buf,connectToCluster,NULL,NULL,"Use Defaults",
				 (gpointer) pos);
}

void
iterativeConnect()
{
	char	buf[1024];
	if (g_slist_length(thePendingHosts) < 1) return;
	char *host = (char *) g_list_nth_data((GList *) thePendingHosts,0);
	sprintf(buf,"Authentication Info for '%s'",host);
	get_passwordentry_dialog(buf,connectToCluster,NULL,NULL,"Use Defaults",
				 (gpointer) 0);
}
