/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     This handles the inter-panel communications mechanism.
 *
 */

#include <stdlib.h>
#include <string.h>
#include "Gessie.h"
#include "Wobject.h"
#include "Broker.h"

static char	*toplevel_name = "VACM";

void
gsSetBrokerToplevelName(char *str) 
{
	toplevel_name = strdup(str);
}

/*
 *	Return a list of panels named 'str'.
 */

GSList *
gsGetPanelList(Wobject *p,char *str) 
{
	Wobject *wobj;
	GSList	*tmp = NULL;

	if (str == NULL) return(NULL);
        Panel	*top = (Panel *) gsSearchPanelsUpward((Panel *) p,toplevel_name);
	if (top == NULL) return(NULL);
        for (unsigned int i = 0; i < g_slist_length(top->listeners); i++) {
                wobj = (Wobject *) g_list_nth_data((GList *) top->listeners,i);
		if (wobj == NULL) continue;
		if (!strcmp(wobj->name(),str)) {
        		tmp = g_slist_append(tmp,(gpointer) wobj);
		}
	}
	return(tmp);
}

void
removePanelFromBrokerList(void *gp)
{
	Panel *p = (Panel *) gp;
	Panel *top = (Panel *) gsSearchPanelsUpward(p,toplevel_name);
	if (top == NULL) {
//		fprintf(stderr,"didn't find my top level (%s)!\n",toplevel_name);
		return;
	}
	top->listeners = g_slist_remove(top->listeners,(gpointer) p);
}

void
addPanelToBrokerList(void *gp)
{
	Panel *p = (Panel *) gp;

		/*------------------------------------------------------*
		 *	        DANGER, DANGER, DANGER!!!!!!		*
		 *------------------------------------------------------* 
		 *	This is wrong! The *Control's should be		*
		 *	subclasses of GenericControl, but I am		*
		 *	too lazy to do that now.			*
		 *------------------------------------------------------*/
	Panel *top = (Panel *) gsSearchPanelsUpward(p,toplevel_name);
	if (top == NULL) {
//		fprintf(stderr,"didn't find my top level (%s)!\n",toplevel_name);
		return;
	}
	top->listeners = g_slist_append(top->listeners,(gpointer) p);
}

void
gsBroadcastBrokerMsg(Wobject *originator,char *msg,gpointer data) 
{
	Wobject	*wobj;
	Panel *top = (Panel *) gsSearchPanelsUpward((Panel *) originator,
							    toplevel_name);
	if (top == NULL) {
//		fprintf(stderr,"sorry can't broadcast msg!\n");
		return;
	}
        for (unsigned int i = 0; i < g_slist_length(top->listeners); i++) {
                wobj = (Wobject *) g_list_nth_data((GList *) top->listeners,i);
		if (wobj == NULL) continue;
		wobj->interested(msg,data);
	}
}
