/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     Handles the aggregation of multiple physical clusters in
 *		a "metacluster".
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include "node.h"
#include "dialogs.h"
#include "question.xpm"
#include "hoover.h"

static void get_metaclusterinfo_dialog(gchar *msg,
		       	   	       void (*callback)(void *handle,
							gpointer clientData,
					    		gpointer callData),
		       	   	       gpointer clientData);

typedef struct {
	void		(*callback)(void *handle,gpointer clientData,
				    gpointer callData);
	gpointer        clientData;
	gpointer        callData;
	GtkWidget       *dialog;
} MetaDialogSquirrel;

/*----------------------------------------------------------------------*
 *				ADD META-CLUSTER			*
 *----------------------------------------------------------------------*/

static void
doTheMetaClusterAddNodes(void * /*handle*/,gpointer clientData,gpointer callData)
{
        Node    *node = (Node *) clientData;
        Node    *childNode;
        Node    *newNode;
        GtkWidget       *table = (GtkWidget *) callData;
        GtkWidget       *item;
        GList   *childList;

        childList = gtk_container_children((GtkContainer *) table);
        for (GList *tmp = childList; tmp != NULL; tmp=tmp->next) {
                item = (GtkWidget *) tmp->data;
                if (gtk_toggle_button_get_active((GtkToggleButton *) item)) {
                        childNode = (Node *) gtk_object_get_data(GTK_OBJECT(item),"node");
                        newNode = copyNode(childNode,node);
			node->addToSubtree(newNode->node_item);
                        node->updateVisual();
                        gtk_widget_show(newNode->node_item);
                }
        }
}

void
doTheMetaClusterAdd(char *metaname,gpointer /*clientData*/,gpointer /*callData*/)
{
	Node            *node = NULL;
	char            buffer[1024];

	/*
	 *      Put together the node.
	 */
	sprintf(buffer,"Metacluster %s",metaname);
	node = new Node(TYPE_METACLUSTER,buffer,NULL,new ConnectInfo(metaname));
	theNodeList = g_slist_append(theNodeList, (gpointer) node);
	node->connection->position = g_slist_length(theNodeList) - 1;
	gtk_tree_append(GTK_TREE(theList), node->node_item);
	gtk_widget_show(node->node_item);
//	tmp->node = node;
//	gtk_tree_item_expand((GtkTreeItem *) node->node_item);
	get_metaclusterinfo_dialog("Select Clusters for Meta-Cluster",
				   doTheMetaClusterAddNodes,(gpointer) node);
}

/*----------------------------------------------------------------------*
 *		    ADD META-CLUSTER NODES DIALOG			*
 *----------------------------------------------------------------------*/

static void
gottaMetaClusterInfoCancel(GtkWidget *,gpointer data)
{
	GtkWidget       *dialog;
	MetaDialogSquirrel *tmp = (MetaDialogSquirrel *) data;
	dialog = tmp->dialog;
	free(tmp);
	gtk_widget_destroy(dialog);
}

static void
gottaMetaClusterInfoAdd(GtkWidget *,gpointer data)
{
	GtkWidget       *dialog;
	MetaDialogSquirrel *tmp = (MetaDialogSquirrel *) data;

	dialog = tmp->dialog;
	if (tmp->callback) {
		tmp->callback((void *) tmp->clientData,tmp->clientData,tmp->callData);
	}

	/*
	 *      clean up and get out
	 */
	free(tmp);
	gtk_widget_destroy(dialog);
}

static void 
get_metaclusterinfo_dialog(gchar *msg,
		       	   void (*callback)(void *handle,gpointer clientData,
					    gpointer callData),
		       	   gpointer clientData) 
{
	GtkWidget	*dialog;
	GtkWidget	*hbox;
	GtkWidget	*table;
	GtkWidget	*label;
	GtkWidget	*icon;
	GtkWidget	*add_button;
	GtkWidget	*cancel_button;
	GtkWidget	*scroller;
	GtkWidget	*item;
	GtkWidget	*elt_table;
	MetaDialogSquirrel	*tmp;
	int	vPos = 0;
	Node	*ctmp;
	int	cnt;

	cnt = numClusters();
	if (cnt == 0) {
		informational_dialog("No Clusters to add to Meta-Cluster!");
		return;
	}

	/* Create the widgets */

	dialog = gtk_dialog_new();
	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog)->vbox),5);

	/*------------------------------------------------------*
	 *			Top portion			*
	 *------------------------------------------------------*/

        hbox = gtk_hbox_new(FALSE /*TRUE*/, 3);
	gtk_box_set_spacing(GTK_BOX(hbox),10);
        gtk_container_set_border_width (GTK_CONTAINER (hbox), 0);
        gtk_widget_show (hbox);

	icon = getIcon((gchar **) question_xpm);
	gtk_container_add(GTK_CONTAINER(hbox),icon);

	label = gtk_label_new(msg);
	gtk_container_add(GTK_CONTAINER(hbox),label);

	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox),hbox);

	/*------------------------------------------------------*
	 *			Data portion			*
	 *------------------------------------------------------*/

        table = gtk_table_new(2 /*rows*/,1 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) table,5);
	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), table);


	scroller = gtk_scrolled_window_new(NULL,NULL);
       	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroller),
                                       GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
         /* The dialog window is created with a vbox packed into it. */                                                              
        gtk_table_attach_defaults(GTK_TABLE(table),scroller,0,1,vPos,vPos+1);
	vPos++;
	gtk_widget_show(scroller); 

	/* create a table of 10 by 10 squares. */
	elt_table = gtk_table_new(1,cnt,FALSE);

	/* set the spacing to 10 on x and 10 on y */
//	gtk_table_set_row_spacings(GTK_TABLE (elt_table), 10);
//	gtk_table_set_col_spacings(GTK_TABLE (elt_table), 10);

	/* pack the table into the scrolled window */
	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(scroller),
					      elt_table);
	gtk_widget_show(elt_table);

        for (unsigned int i = 0; i < g_slist_length(theNodeList); i++) {
                ctmp = (Node *) g_list_nth_data((GList *) theNodeList,i);
		if (ctmp->type != TYPE_CLUSTER) continue;
		item = gtk_check_button_new_with_label(ctmp->name);
		gtk_object_set_data(GTK_OBJECT(item),"node",(void*) ctmp);
		gtk_table_attach_defaults(GTK_TABLE(elt_table),item,0,1,i,i+1);
		gtk_widget_show(item);
        }

	/*------------------------------------------------------*
	 *			Button portion			*
	 *------------------------------------------------------*/

	/*
	 *	Stash away some stuff for later
	 */
	tmp = (MetaDialogSquirrel *) malloc(sizeof(MetaDialogSquirrel));
	tmp->dialog     = dialog;
	tmp->callback   = callback;
	tmp->clientData = clientData;
	tmp->callData	= elt_table;

	add_button = gtk_button_new_with_label("Apply");
	gtk_signal_connect(GTK_OBJECT(add_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaMetaClusterInfoAdd), 
			   (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  add_button);
	GTK_WIDGET_SET_FLAGS(add_button,GTK_CAN_DEFAULT);
	gtk_widget_grab_default(add_button);

	cancel_button = gtk_button_new_with_label("Cancel");
	gtk_signal_connect(GTK_OBJECT(cancel_button), "clicked",
		       	   GTK_SIGNAL_FUNC (gottaMetaClusterInfoCancel),
			   (gpointer) tmp);
	gtk_container_add(GTK_CONTAINER (GTK_DIALOG(dialog)->action_area),
	       		  cancel_button);

	/* Add the label, and show everything we've added to the dialog. */
	gtk_window_set_position(GTK_WINDOW(dialog),GTK_WIN_POS_MOUSE);
	gtk_widget_show_all(dialog);
}
