/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     Crack and handle stuff surrounding the cluster node lists.
 *
 * Notes:	The handleNodeListMessage() needs to handle the
 *		removal of nodes.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <gtk/gtk.h>
#include "node.h"
#include "EMPservices.h"
#include "RSHservices.h"
#include "input_queue.h"

extern int	checkTreeStates(gpointer data);

static void
crackNodeEntry(void *handle,char *msg,gboolean search)
{
	Node	*node;
	Node	*rootNode = handleToRootNode(handle);
	ConnectInfo	*con = handleToConnectInfo(handle);

//	fprintf(stderr,"#####crackNodeEntry(%s)\n",msg);
	assert(con != NULL);
	if (con) {
		con->node = rootNode;
	}
	(void) strtok(msg,":");			// skip NODELIST
	char *nodeid = strtok(NULL,":");	// the node id
	char *typeName  = strtok(NULL,":");	// name of the type
	char *typevalue = strtok(NULL,":");	// value of the variable

	/*
	 *	Go through the children and see if something
	 *	matches and then we can throw away this entry.
	 *	We may, however, wish to update the matching
	 *	node from the new info.
	 */

	if (search) {
		Node	*child;
		for (unsigned int i = 0; i < g_slist_length(rootNode->childList); i++) {
			child = (Node *) g_list_nth_data((GList *) rootNode->childList,i);
			if (!strcmp(nodeid,child->name)) {
				child->misc_flags |= N_MISC_TRAVERSED;
				return;
			}
		}
	}

	NodeType nT = TYPE_UNKNOWN_NODE;
	/*
	 *	see if we have it in the variable.
	 */
	if (   (typeName  != NULL)
	    && (typevalue != NULL)
	    && !strcasecmp(typeName,"type")) {
		nT = nodeTypeFromName(typevalue);
	}

	/*----------------------------------------------*
	 *		Finally, add the node.		*
	 *----------------------------------------------*/

	node = new Node(nT,nodeid,rootNode,con);

	theNodeList = g_slist_append(theNodeList, (gpointer) node);
	gtk_tree_append(GTK_TREE(rootNode->getSubtree()), node->node_item);
	node->updateVisual();
	gtk_widget_show(node->node_item);
	node->misc_flags |= N_MISC_TRAVERSED;

	if (rootNode) {
		rootNode->expandSubtree();	// default to open
	}
}

void
handleNodeListMessage(InputEntry *entry,gpointer clientData)
{
	void	*handle	  = (void *) clientData;
	Node	*rootNode = handleToRootNode(handle);
	int	i;
	Node	*child;

	/*
	 *	Go through all the nodes in list list and mark them
	 *	as not having been traversed so that we can see if
	 *	anything has been missed and is likely deleted.
	 */
	for (i = 0; i < (int) g_slist_length(rootNode->childList); i++) {
		child = (Node *) g_list_nth_data((GList *) rootNode->childList,i);
		child->misc_flags &= ~N_MISC_TRAVERSED;
	}

	/*
	 *	Go through the entries.
	 */
	for (InputEntry *tmp = entry; tmp; tmp = tmp->next) {
		crackNodeEntry(handle,tmp->string,TRUE /*FALSE*/);
        }
	checkTreeStates(NULL);

	/*
	 *	Go through the children and see if anything wasn't
	 *	included in the list.
	 */
	for (i = 0; i < (int) g_slist_length(rootNode->childList); i++) {
		child = (Node *) g_list_nth_data((GList *) rootNode->childList,i);
		if (!(child->misc_flags & N_MISC_TRAVERSED)) {
			if (globalVerboseMode >= 2) {
				fprintf(stderr,"handleNodeListMessage() - node '%s' was removed!\n",child->name);
			}
			delete child;
		}
	}
}
