/*
 * Copyright (C) 1999 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     Most of the node property setting dialogs and cluster
 *		lists.
 *
 */

#include <stdio.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include "node.h"
#include "gtkhacks.h"

typedef struct nodePropDialogSquirrel_t {
	Node		*node;
	GtkWidget	*name_entry;
	GtkWidget	*emp_entry;
	GtkWidget	*console_entry;
	GtkWidget	*pass_entry;
	GtkWidget	*eth0_entry;
} NodePropDialogSquirrel;

/*----------------------------------------------------------------------*
 *				PROPERTIES				*
 *----------------------------------------------------------------------*/

static void
properties_cancel_cb(gpointer data,GtkWidget *)
{
	NodePropDialogSquirrel	*tmp = (NodePropDialogSquirrel *) data;
	Node	*node = tmp->node;

	gtk_widget_hide(node->prop_win);
	/*
	 *	NOTE!!!! Need to reset the fields
	 */
	gtk_entry_set_text(GTK_ENTRY(tmp->name_entry),node->name);
	gtk_entry_set_text(GTK_ENTRY(tmp->emp_entry),(node->emp_port == NULL) 
							? "" : node->emp_port);
	gtk_entry_set_text(GTK_ENTRY(tmp->console_entry),(node->console == NULL) 
							? "" : node->console);
	gtk_entry_set_text(GTK_ENTRY(tmp->eth0_entry),(node->ip == NULL) 
							? "" : node->ip);
}

static void     
handleNodeConfiguredMessage(InputEntry *,gpointer)
{
}

static void
properties_apply_cb(gpointer data,GtkWidget *)
{
	NodePropDialogSquirrel	*tmp = (NodePropDialogSquirrel *) data;
	Node	*node = tmp->node;
	gchar	*txt;
	GtkWidget *label;

	/*
	 *	Change the name
	 */
	txt = gtk_entry_get_text((GtkEntry *) tmp->name_entry);
	if (txt != NULL && strlen(txt)) {
		if (node->name) free(node->name);
		node->name = strdup(txt);
		label = (GtkWidget *) gtk_tree_item_get_label(node->node_item);
	}
	
	/*
	 *	Change the emp port
	 */
	txt = gtk_entry_get_text((GtkEntry *) tmp->emp_entry);
	if (txt != NULL && strlen(txt)) {
		if (node->emp_port) free(node->emp_port);
		node->emp_port = strdup(txt);
	}

	/*
	 *	Change the console port
	 */
	txt = gtk_entry_get_text((GtkEntry *) tmp->console_entry);
	if (txt != NULL && strlen(txt)) {
		if (node->console) free(node->console);
		node->console = strdup(txt);
	}

	/*
	 *	Change the password
	 */
	txt = gtk_entry_get_text((GtkEntry *) tmp->pass_entry);
	if (txt != NULL && strlen(txt)) {
		if (node->password) free(node->password);
		node->password = strdup(txt);
	}
	/*
	 *	Change the IP address
	 */
	txt = gtk_entry_get_text((GtkEntry *) tmp->eth0_entry);
	if (txt != NULL && strlen(txt)) {
		if (node->ip) free(node->ip);
		node->ip = strdup(txt);
	}
	/*
	 *	Fiddle with the configuration
	 */
	node->services->doNodeConfigure((InputEntryProc) NULL,node->name,
					node->emp_port,node->password,node->ip);
	node->updateVisual();		/* show any changes made */
	gtk_widget_hide(node->prop_win);
}

static void
prop_delete_event(GtkWidget *,GdkEvent *,gpointer data)
{
	NodePropDialogSquirrel	*tmp = (NodePropDialogSquirrel *) data;
	Node	*node = tmp->node;

	gtk_widget_hide(node->prop_win);
}

void
createProperties(Node *node,gpointer /*data*/)
{
	NodePropDialogSquirrel	*tmp;
	char	buffer[256];
	GtkWidget	*table;
	GtkWidget	*hbox;
	GtkWidget		*apply;
	GtkWidget		*cancel;
	GtkWidget	*label;
	GtkWidget	*entry;
	int	vertPos = 0;


	if (node == NULL) return;
	if (!node->singularTypeP()) return;
	if (node->prop_win != NULL) {
		/*
		 *	Already present, do something about it
		 */
		gtk_widget_show(node->prop_win);
		return;
	}

        tmp = (NodePropDialogSquirrel *) malloc(sizeof(NodePropDialogSquirrel));
	tmp->node = node;

	node->prop_win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	sprintf(buffer,"Properties for \'%s\'",node->name);
	gtk_window_set_title(GTK_WINDOW(node->prop_win), buffer);
	gtk_signal_connect(GTK_OBJECT(node->prop_win), "delete_event",
			   GTK_SIGNAL_FUNC(prop_delete_event), (gpointer) tmp);
	gtk_signal_connect(GTK_OBJECT(node->prop_win), "destroy",
			   GTK_SIGNAL_FUNC(prop_delete_event), (gpointer) tmp);
	gtk_widget_show(node->prop_win);

	table = gtk_table_new(7 /*rows*/,2 /*cols*/,FALSE);
	gtk_table_set_row_spacings((GtkTable *) table,5);
	gtk_container_add(GTK_CONTAINER(node->prop_win), table);


#define	PROP_LABEL(s)							\
		label = gtk_label_new(s);				\
		gtk_label_set_justify((GtkLabel *) label,GTK_JUSTIFY_LEFT); \
		gtk_table_attach(GTK_TABLE(table),label,0,1,vertPos,	\
				 vertPos+1,GTK_SHRINK, GTK_SHRINK, 0,0)

	/*
	 *	NODE LABEL
	 */
	PROP_LABEL("Node Label ");

	tmp->name_entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_editable(GTK_ENTRY(tmp->name_entry), TRUE);
	if (node->name)
		gtk_entry_set_text(GTK_ENTRY(tmp->name_entry), node->name);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp->name_entry,1,2,vertPos,
				  vertPos+1);
	gtk_widget_show(tmp->name_entry);
	gtk_widget_show(label);
	vertPos++;

	/*
	 *	NODE PASSWORD
	 */
	if (node->type == TYPE_MSC_NODE) {
		PROP_LABEL("MSC Password ");
	}
	else	{
		PROP_LABEL("EMP Password ");
	}

	tmp->pass_entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_editable(GTK_ENTRY(tmp->pass_entry), TRUE);
	if (node->password)
		gtk_entry_set_text(GTK_ENTRY(tmp->pass_entry), node->password);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp->pass_entry,1,2,vertPos,
				  vertPos+1);
	gtk_widget_show(tmp->pass_entry);
	gtk_widget_show(label);
	vertPos++;

	if (node->type == TYPE_EMP_NODE) {
		/*
		 *	NODE SERIAL NUMBER
		 */
		PROP_LABEL("Node Serial # ");

		entry = gtk_entry_new_with_max_length(32);
		gtk_entry_set_editable(GTK_ENTRY(entry), FALSE);
		if (node->serial_number)
			gtk_entry_set_text(GTK_ENTRY(entry),node->serial_number);
		gtk_table_attach_defaults(GTK_TABLE(table),entry,1,2,vertPos,
					  vertPos+1);
		gtk_widget_show(entry);
		gtk_widget_show(label);
		vertPos++;
	}

	/*
	 *	NODE EMP PORT
	 */
	if (node->type == TYPE_MSC_NODE) {
		PROP_LABEL("Node Aux port ");
	}
	else	{
		PROP_LABEL("Node EMP port ");
	}

	tmp->emp_entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_editable(GTK_ENTRY(tmp->emp_entry), TRUE);
	if (node->emp_port)
		gtk_entry_set_text(GTK_ENTRY(tmp->emp_entry), node->emp_port);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp->emp_entry,1,2,vertPos,
				  vertPos+1);
	gtk_widget_show(tmp->emp_entry);
	gtk_widget_show(label);
	vertPos++;
	
	/*
	 *	NODE CONSOLE PORT
	 */
	PROP_LABEL("Node Console port ");

	tmp->console_entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_editable(GTK_ENTRY(tmp->console_entry), TRUE);
	if (node->console)
		gtk_entry_set_text(GTK_ENTRY(tmp->console_entry), node->console);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp->console_entry,1,2,vertPos,
				  vertPos+1);
	gtk_widget_show(tmp->console_entry);
	gtk_widget_show(label);
	vertPos++;

	/*
	 *	NODE ETH0 PORT
	 */
	PROP_LABEL("Node eth0 addr ");

	tmp->eth0_entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_editable(GTK_ENTRY(tmp->eth0_entry), TRUE);
	if (node->ip)
		gtk_entry_set_text(GTK_ENTRY(tmp->eth0_entry), node->ip);
	gtk_table_attach_defaults(GTK_TABLE(table),tmp->eth0_entry,1,2,vertPos,
				  vertPos+1);
	gtk_widget_show(tmp->eth0_entry);
	gtk_widget_show(label);
	vertPos++;

	/*
	 *	NODE STATE
	 */
	PROP_LABEL("Node State ");

	entry = gtk_entry_new_with_max_length(32);
	gtk_entry_set_editable(GTK_ENTRY(entry),FALSE);
	gtk_entry_set_text(GTK_ENTRY(entry),node->stateString());
	gtk_table_attach_defaults(GTK_TABLE(table),entry,1,2,vertPos,vertPos+1);
	gtk_widget_show(entry);
	gtk_widget_show(label);
	vertPos++;

	/*--------------------------------------------------------------*
	 *			Bottom Buttons				*
	 *--------------------------------------------------------------*/

	gtk_table_attach_defaults(GTK_TABLE(table),gtk_hseparator_new(),0,2,
				  vertPos,vertPos+1);
	vertPos++;

        hbox = gtk_hbox_new(FALSE /*TRUE*/,3);
        gtk_container_set_border_width(GTK_CONTAINER (hbox),0);
        gtk_widget_show(hbox);
        gtk_table_attach(GTK_TABLE(table),hbox,0,2,vertPos,vertPos+1,
                         (enum GtkAttachOptions)(GTK_FILL|GTK_EXPAND),
			 GTK_SHRINK,0,0);

	apply = gtk_button_new_with_label("Apply");
	gtk_container_add(GTK_CONTAINER(hbox),apply);
	gtk_container_set_border_width(GTK_CONTAINER(apply),2);
	gtk_signal_connect_object(GTK_OBJECT(apply),"clicked",
				  GTK_SIGNAL_FUNC(properties_apply_cb),
				  (GtkObject *) tmp);
	gtk_widget_show(apply);

	cancel = gtk_button_new_with_label("Cancel");
	gtk_container_add(GTK_CONTAINER(hbox),cancel);
	gtk_container_set_border_width(GTK_CONTAINER(cancel),2);
	gtk_signal_connect_object(GTK_OBJECT(cancel),"clicked",
				  GTK_SIGNAL_FUNC(properties_cancel_cb),
				  (GtkObject *) tmp);
	gtk_widget_show(cancel);

	gtk_table_set_row_spacing((GtkTable *) table,3,10);
	gtk_widget_show(table);
}

/*----------------------------------------------------------------------*
 *			CLUSTER PROPERTIES TABLE			*
 *----------------------------------------------------------------------*/

void
checkClusterPropertyStates(Node *changedNode)
{
	int	theRow;
	Node	*tmp;
	Node	*node;
	char	*newState;
        Node	*theNode;

        for (unsigned int i = 0; i < g_slist_length(theNodeList); i++) {
                theNode = (Node *) g_list_nth_data((GList *) theNodeList,i);
                if (theNode->connection->handle == NULL)	continue;
		if ((node = theNode) == NULL)		continue;
		if (node->type != TYPE_CLUSTER)		continue;
		if (node->sel_log == NULL) 		continue;
                for (unsigned int j = 0; i < g_slist_length(node->childList); j++) {
                        tmp = (Node *) g_list_nth_data((GList *) node->childList,j);
			if (tmp != changedNode) continue;
			newState = strdup(tmp->stateString());
			theRow   = gtk_clist_find_row_from_data(GTK_CLIST(node->sel_log),
								tmp);
			gtk_clist_set_text(GTK_CLIST(node->sel_log),theRow,6,
					   newState);
		}
        }
}
