/*
 * Copyright (C) 2000 Silicon Graphics, Inc.  All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of version 2 of the GNU General Public License as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE.  Further, any license provided herein,
 * whether implied or otherwise, is limited to this program in accordance
 * with the express provisions of the GNU General Public License.  Patent
 * licenses, if any, provided herein do not apply to combinations of this
 * program with other product or programs, or any other product whatsoever.
 * This program is distributed without any warranty that the program is
 * delivered free of the rightful claim of any third person by way of
 * infringement or the like.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 */

/*
 * Author:      Dean Johnson (dtj@sgi.com)
 *
 * Purpose:     This is most of the node specific code.
 *
 * Notes:       I am in the process of further abstracting the concept
 *              of 'node' and will have subclasses such as "EMP_NODE"
 *              and eventually other types.
 *
 */

#include <stdio.h>
#include <strings.h>
#include "hoover.h"

struct nodeTypeRecs {
	NodeType	type;
	char	*mnemonic;
	char	*description;
};

static struct nodeTypeRecs	recs[] = {
	{ TYPE_UNKNOWN_NODE,	"UNK",	"bad or unknown type"		},
	{ TYPE_EMP_NODE,	"EMP",	"Intel EMP/IPMI endowed node"	},
	{ TYPE_MSC_NODE,	"MSC",	"SGI MSC endowed node"		},
	{ TYPE_RSH_NODE,        "RSH",	"'rsh' accessible node"		},
	{ TYPE_CLUSTER,		"CLU",	""				},
	{ TYPE_ALIAS,           "ALI",	"cluster node in metacluster"	},
	{ TYPE_METACLUSTER,	"MET",	""				},
};

static int	numrecs = sizeof(recs) / sizeof(recs[0]);

NodeType
nodeTypeFromName(char *nm)
{
	for (int i = 0; i < numrecs; i++) {
		if (!strcasecmp(nm,recs[i].mnemonic)) return(recs[i].type);
	}
	return(TYPE_UNKNOWN_NODE);
}

char *
nodeNameFromType(NodeType nt)
{
	for (int i = 0; i < numrecs; i++) {
		if (nt == recs[i].type) return(recs[i].mnemonic);
	}
	return("UNK");
}
