%define name    vacm
%define ver     2.0.5a
%define rel     1
%define prefix  /usr

Summary: system for monitoring and management of a cluster of nodes
Name: %{name}
Version: %{ver}
Release: %{rel}
Prefix: %{prefix}
Copyright: GPL
Group: System Environment/Daemons
Packager: Dean Johnson (dtj@sgi.com), Michael Jennings <mej@valinux.com>, The Rasterman (Carsten Haitzler) <raster@valinux.com> and Zac Sprackett <zsprackett@valinux.com> 
Source: http://download.sourceforge.net/vacm/vacm-%{ver}.tar.gz
URL: 	http://vacm.sourceforge.net/
Obsoletes: VACM vacm vacm-ssl
Buildroot: /var/tmp/%{name}-%{ver}-root

%description
VACM (Pronounced VaKuum) is a client / server system allowing
monitoring and management of a cluster of nodes equipped with
Intel(TM)'s Intelligent Platform Management Interface (IPMI).
Management capabilities include device power control, nvram system
event log download, hardware reset, and remote bios console.
Monitoring capabilities include power system monitoring, chassis
intrusion monitoring, fan status monitoring, and O/S watchdog
monitoring. Since the IPMI is implemented in hardware running on
separate microcontrollers, monitoring a node does *not* impact on the
monitored node's performance.  Furthermore, VACM allows monitoring and
management of a node from a remote station (via TCP/IP).

%package devel
Summary: VACM devel RPM
Group: System Environment/Daemons
%description devel
Header files for developing VACM modules

%package node
Summary: VACM client node RPM
Group: System Environment/Daemons
%description node
Package to be installed on VACM client nodes.  It contains daemons
for use with various VACM modules for additional monitoring
functionality.

%package flim
Summary: Flim GUI for VACM
Group: System Environment/Daemons
Requires: gtk+ >= 1.2.6
Requires: XFree86 >= 3.0.0
Requires: /usr/X11R6/bin/xterm
Requires: vacm-clientlib >= 2.0
Requires: vacm-sercon >= 2.0
%description flim
Flim is a Graphical User Interface (GUI) for VACM which provides
easy interaction with the nexxus through several plugin modules.

%package hoover
Summary: Hoover GUI for VACM
Group: System Environment/Daemons
Requires: gtk+ >= 1.2.6
Requires: vacm-clientlib >= 2.0
%description hoover
Hoover is a Graphical User Interface (GUI) for VACM.

%package sercon
Summary: Serial Console Terminal program
Group: System Environment/Daemons
Requires: vacm-clientlib >= 2.0
%description sercon
Command-line serial console terminal program to remotely access consoles of
nodes on a VACM cluster

%package clientlib
Summary: VACM client library RPM
Group: System Environment/Daemons
Obsoletes: vacm-lib
%description clientlib
VACM client library required for VACM clients

%package vash
Summary: VACM cmd-line client RPM
Group: System Environment/Daemons
Requires: vacm-clientlib >= 2.0
%description vash
VACM command line client for scripting and low level command-line access

%package doc
Summary: VACM documentation
Group: System Environment/Daemons
%description doc
VACM documentation.  This package should contain SGML, postscript and HTML
versions of the docs.

%prep
%setup -q -n vacm-%{ver}

%build
# Optimize compiles
if [ ! -z "`echo -n ${RPM_OPT_FLAGS} | grep pentium`" ]; then
  if [ ! -z "`which pgcc`" ]; then
    CC="pgcc"
  fi
fi
CFLAGS="${RPM_OPT_FLAGS} -O3"
export CFLAGS
CXXFLAGS="${RPM_OPT_FLAGS} -O3"
export CXXFLAGS
./configure --prefix=%{prefix}
make

%install
make DESTDIR=$RPM_BUILD_ROOT gnulocaledir=$RPM_BUILD_ROOT%{prefix}/share/locale install

find $RPM_BUILD_ROOT/%{prefix}/doc/ -type f -print |\
	sed -e "s@^$RPM_BUILD_ROOT@@g" > doc-file.lst

mkdir -p $RPM_BUILD_ROOT%{prefix}/sbin
install -m 755 $RPM_BUILD_ROOT%{prefix}/lib/vacm/exports/* $RPM_BUILD_ROOT%{prefix}/sbin

mkdir -p $RPM_BUILD_ROOT/etc/rc.d/init.d
install -m 755 packaging/RedHat/vacm.init $RPM_BUILD_ROOT/etc/rc.d/init.d/vacm
install -m 755 packaging/RedHat/node.init $RPM_BUILD_ROOT/etc/rc.d/init.d/vacm-node

mkdir -p $RPM_BUILD_ROOT/etc/logrotate.d
install -m 644 packaging/RedHat/vacm-logrotate $RPM_BUILD_ROOT/etc/logrotate.d/vacm

mkdir -p $RPM_BUILD_ROOT/etc/sysconfig
install -m 644 packaging/RedHat/vacm.sysconfig $RPM_BUILD_ROOT/etc/sysconfig/vacm

# Catalog
cd $RPM_BUILD_ROOT
find . -print

%clean
rm -rf $RPM_BUILD_ROOT
rm -f doc-file.lst

%post
[ -x /etc/rc.d/init.d/vacm ] && /sbin/chkconfig --add vacm

%preun
[ -x /etc/rc.d/init.d/vacm ] && /sbin/chkconfig --del vacm
	
%post node
[ -x /etc/rc.d/init.d/vacm-node ] && /sbin/chkconfig --add vacm-node
[ -d /dev/ipmi ] || rm -f /dev/ipmi 
[ -r /dev/ipmi ] || mkdir /dev/ipmi
[ -c /dev/ipmi/kcs ] || mknod /dev/ipmi/kcs c 10 173

%preun node
[ -x /etc/rc.d/init.d/vacm-node ] && /sbin/chkconfig --del vacm-node

%files
%defattr(-,root,root)
%{prefix}/lib/libloose*
%{prefix}/lib/vacm/modules/*
%{prefix}/lib/vacm/exports/*
%{prefix}/bin/nexxus
/etc/rc.d/init.d/vacm
%config %{prefix}/lib/vacm/vacm_configuration
%config /etc/sysconfig/vacm

%files devel
%defattr(-,root,root)
%{prefix}/include/*

%files node
%defattr(-,root,root)
%{prefix}/sbin/*
/etc/rc.d/init.d/vacm-node

%files flim
%defattr(-,root,root)
%{prefix}/bin/flim
%{prefix}/lib/flim/*
%{prefix}/share/flim/*
%{prefix}/share/locale/*/LC_MESSAGES/flim.mo

%files hoover
%defattr(-,root,root)
%{prefix}/bin/hoover
%{prefix}/lib/libgessie*
%{prefix}/man/man8/hoover.8

%files sercon
%defattr(-,root,root)
%{prefix}/bin/sercon_terminal

%files clientlib
%defattr(-,root,root)
%{prefix}/lib/libvacmclient*

%files vash
%defattr(-,root,root)
%{prefix}/bin/vash

%files doc -f doc-file.lst
%defattr(-,root,root)
